function [EMGEns] = alignSortEMGData(DataMech,DataEMG,MechStretchIdxs,Settings)
%alignSortEMGData Align and sort the EMG data
%   Analyze the EMG data in the time-domain by synchronizing EMG with 
%   mechanical data, and aligning data at stretch/perturbation onsets and 
%   sorting them into ensembles linked to various perturbation characteristics.
%
%	SYNTAX
%   [EMGEns] = alignSortEMGData(DataMech,DataEMG,MechStretchIdxs,Settings)
%
%	INPUTS
%	DataMech        = Selection of (bio)mechanical signals (struct)
%   DataEMG         = Selection of EMG signals (struct)
%   MechStretchIdxs = Sets of stretch/perturbation onset indices in (bio)mechanical data (struct)
%   Settings        = Overview of all processing settings (struct)
%
%   OUTPUT
%   EMGEns = Aligned and sorted EMG signals (struct)

    % Create processing settings and time vector
    segStartEMG = round(Settings.tSegStartEMG/DataEMG.dt); % [-] Samples before 
    segEndEMG = round(Settings.tSegEndEMG/DataEMG.dt); % [-] Samples after
    EMGEns.tSeg = (0:segEndEMG-1)'*DataEMG.dt;

    for ii = 1:numel(Settings.sortFields)
        % Cross-link all mechanical-based indices to EMG-based indices
         % Get all mechanical index times
        mechStretchTimes = DataMech.t(MechStretchIdxs.(Settings.sortFields{ii}));
        emgStretchIdxs = zeros(numel(mechStretchTimes),1);
         % Set maximum search length based on next stretch time
        checkLength = round((mechStretchTimes(1) - DataEMG.t(1))/DataEMG.dt) + 100;
         % Find corresponding EMG time with minimal absolute difference
        [~,emgStretchIdxs(1)] = min(abs(DataEMG.t(1:checkLength) - mechStretchTimes(1)));
         % Repeat process for all mechanical stretch times
        for jj = 2:numel(mechStretchTimes)
            checkLength = round((mechStretchTimes(jj) - DataEMG.t(emgStretchIdxs(jj-1)))/DataEMG.dt) + 100;
            [~,emgStretchIdxs(jj)] = min(abs(DataEMG.t((1:checkLength) + emgStretchIdxs(jj-1)) - mechStretchTimes(jj)));
            emgStretchIdxs(jj) = emgStretchIdxs(jj) + emgStretchIdxs(jj-1);
        end
         % Store the EMG-based indices
        EMGEns.StretchIdxs.(Settings.sortFields{ii}) = emgStretchIdxs;
            
        % Pre-allocate ensemble data structure
        EMGEns.(Settings.sortFields{ii}) = struct('solBack',{},'taBack',{},'glBack',{},'gmBack',{},...
                                                      'solReflex',{},'taReflex',{},'glReflex',{},'gmReflex',{},'syncReflex',{});
        % Retrieve all the seperate EMG responses to all pulses/steps
        EMGEns.(Settings.sortFields{ii}) = getEMGStretchEnsembles(EMGEns.StretchIdxs.(Settings.sortFields{ii}),EMGEns.(Settings.sortFields{ii}),...
            segStartEMG,segEndEMG,DataEMG.sol,DataEMG.ta,DataEMG.gm,DataEMG.gl,DataEMG.syncEMGScaled);
    end
end