function [StretchIdxs] = getStretchIndices(posRef,segStart,segEnd,dtMech)
%getStretchIndices Retrieve mechanical start indices for stretch reflexes
%   Retrieve all the seperate sets of stretch reflex ensemble starting
%   indices within the mechanical data. Function checks based on segment
%   length whether enough data is available before/after index.
%
%   SYNTAX
%   [StretchIdxs] = getStretchIndices(posRef,segStart,segEnd,dtMech)
%
%	INPUTS
%   posRef   = Reference position vector for motor (Mx1 double)
%   segStart = Number of samples before mechIdx at start of segments (double)
%   segEnd   = Number of samples after mechIdx at end of segments (double)
%   dtMech   = Sampling time mechanical data [s] (double)
%
%   OUTPUT
%   StretchIdxs = Sets of stretch/perturbation onset indices (struct)

    % Use reference perturbation signal
    % Rising edges (via block-shaped reference signal)
    dRef = diff(posRef);
    x = find(dRef(1:end-1) > 0);
    % Get all potential perturbation onsets
    pertOnsetIdxs = [1; find(diff(x)>1)+1];
    if (~all(diff(pertOnsetIdxs) == 1))
        % Remove any potential onsets with no next velocity sample larger
        % than 0, i.e. noise
        pertOnsetIdxs = pertOnsetIdxs(any(dRef(x(pertOnsetIdxs) + 1) > 1.5e-6,2));
    end
    nDum = x(pertOnsetIdxs);
    % Remove indices with incomplete data
    nDum = nDum(2:end-1); % First and last index (risk of EMG missing)
    StretchIdxs.rise = nDum((nDum > segStart) & (nDum < (length(posRef)-segEnd))); % Missing Achilles data

    % Falling edges (via block-shaped reference signal)
    x = find(dRef(1:end-1) < 0);
    % Get all potential perturbation onsets
    pertOnsetIdxs = [1; find(diff(x)>1)+1];
    if (~all(diff(pertOnsetIdxs) == 1))
        % Remove any potential onsets with no next velocity sample smaller
        % than 0, i.e. noise
        pertOnsetIdxs = pertOnsetIdxs(any(dRef(x(pertOnsetIdxs) + 1) < -1.5e-6,2));
    end
    nDum = x(pertOnsetIdxs);
    % Remove indices with incomplete data
    nDum = nDum(2:end-1); % First and last index (risk of EMG missing)
    StretchIdxs.fall = nDum((nDum > segStart) & (nDum < (length(posRef)-segEnd))); % Missing Achilles data

    % Find pulses and steps to split
    StretchIdxs.nStep = find(posRef(StretchIdxs.rise+round(0.1/dtMech)) > 0);
    StretchIdxs.nPulse = find(posRef(StretchIdxs.rise+round(0.1/dtMech)) < 0);

    % Get all (post-)pulse/step indices
    StretchIdxs.pulse     = StretchIdxs.rise(StretchIdxs.nPulse);
    StretchIdxs.postPulse = StretchIdxs.rise(StretchIdxs.nPulse(StretchIdxs.nPulse ~= numel(StretchIdxs.rise))+1);
    StretchIdxs.step      = StretchIdxs.rise(StretchIdxs.nStep);
    StretchIdxs.postStep  = StretchIdxs.rise(StretchIdxs.nStep(StretchIdxs.nStep ~= numel(StretchIdxs.rise))+1);

    % Find step indices in 'fall'-direction
    if (StretchIdxs.fall(1) > StretchIdxs.rise(1))
        if (StretchIdxs.fall(end) > StretchIdxs.rise(end))
            StretchIdxs.nStepFall = StretchIdxs.nStep;
        else
            StretchIdxs.nStepFall = StretchIdxs.nStep(1:end-1);
        end
    else
        if (StretchIdxs.fall(end) > StretchIdxs.rise(end))
            StretchIdxs.nStepFall = StretchIdxs.nStep+1;      
        else
            StretchIdxs.nStepFall = StretchIdxs.nStep(1:end-1)+1;
        end
    end
    StretchIdxs.stepFall = StretchIdxs.fall(StretchIdxs.nStepFall);
end