function [EMGData] = interpolateEMGData(EMGData,Settings)
%interpolateEMGData Interpolate missing samples in raw EMG dataset
%   Interpolates missing samples in the raw EMG dataset by repeating the
%   last recorded sample until a new EMG sample is found.
%
%	SYNTAX
%   [EMGData] = interpolateEMGData(EMGData,Settings)
%
%	INPUTS
%   EMGData  = EMG measurement dataset (MxN double)
%   Settings = Overview of all processing settings (struct)
%
%   OUTPUT
%   EMGData  = EMG measurement dataset with interpolated missing samples (MxL double)

    % Check for missing samples based on sawtooth signal
    sawtooth = EMGData(Settings.EMGDataRows(6),:)';
    missIdxs = find((diff(sawtooth) ~= 2) & (diff(sawtooth) ~= -62));
    for ii = numel(missIdxs):-1:1
        % Compute number of missing samples, assuming minimum possible
        % (uncertainty due to modulo 64 sawtooth signal)
        addSamples = (mod(sawtooth(missIdxs(ii)+1) - sawtooth(missIdxs(ii)) - 2,64))/2;
        % Create matrix to add by repeating last recorded sample
        addMatrix = [EMGData(1:size(EMGData,1) ~= Settings.EMGDataRows(6),missIdxs(ii))*ones(1,addSamples); ...
                     mod(sawtooth(missIdxs(ii))+2*(1:addSamples),64)];
        % Insert matrix into EMG dataset
        EMGData = [EMGData(:,1:missIdxs(ii)),addMatrix,EMGData(:,missIdxs(ii)+1:end)];
    end
end