function [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings)
%loadData Load (bio)mechanical and [optionally] EMG data
%   Load all the (bio)mechanical data and [optionally] EMG data available
%   from experiments/simulation. Relevant signals are selected and stored.
%
%	SYNTAX
%   [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings)
%
%	INPUTS
%   dataSet  = (Bio)mechanical data location (string)
%   Settings = Overview of all processing settings (struct)
%
%   OUTPUTS
%	DataMech  = Selection of (bio)mechanical signals (struct)
%   DataEMG   = Selection of EMG signals (struct) [optional]
%   DataSysID = System Identification (seperate intrinsic/reflexive) results (struct)
%   Settings  = Overview of all processing settings (struct)

    % Load (bio)mechanical data
    loadedData = load(dataSet);
    % Time characteristics
    DataMech.dt = 1/loadedData.OutputData.TrialSettings.fsAch; % [s] Achilles Perturbator
    data = loadedData.OutputData.AchData;
    
    % Select (bio)mechanical signals: position/velocity/torque and reference
    DataMech.t = data(:,Settings.mechDataColumns(1));       % [s] Time vector data
    DataMech.posRef  = data(:,Settings.mechDataColumns(2)); % [rad] Position reference
    DataMech.posMeas = data(:,Settings.mechDataColumns(4)); % [rad] Position measured
    DataMech.velRef  = data(:,Settings.mechDataColumns(3)); % [rad/s] Velocity reference
    DataMech.velMeas = data(:,Settings.mechDataColumns(5)); % [rad/s] Velocity measured
    DataMech.tqMeas = data(:,Settings.mechDataColumns(6));  % [Nm] Torque measured
    DataMech.sync = data(:,Settings.mechDataColumns(7));    % [-] Synchronization signal
    
    % Center positions signals around zero with given perturbation amplitude
    DataMech.posRef = DataMech.posRef - DataMech.posRef(1) - Settings.pertAmpl/2;
    DataMech.posMeas = DataMech.posMeas - DataMech.posMeas(1) - Settings.pertAmpl/2;
    if (~all(mod(DataMech.sync,1) == 0)) 
        DataMech.sync = DataMech.sync - DataMech.sync(1) - Settings.pertAmpl/2;
    end
    % Center torque signal around zero (based on session or local torque norm)
    if (isfield(loadedData.OutputData.TrialSettings,'isLocalTqNorm') && (loadedData.OutputData.TrialSettings.isLocalTqNorm))
        DataMech.tqMeas = DataMech.tqMeas - loadedData.OutputData.TrialSettings.initialTorque;
    else
        DataMech.tqMeas = DataMech.tqMeas - loadedData.OutputData.SessionSettings.InitialTorque;
    end
    
    % Load EMG data
    if (Settings.isEMG)
        EMGData = loadedData.OutputData.EMGData;
        DataEMG.rawSawtooth = EMGData(Settings.EMGDataRows(6),:)';
        % Interpolate missing samples in raw EMG dataset
        EMGData = interpolateEMGData(EMGData,Settings);
        % Time characteristics
        DataEMG.dt = 1/loadedData.OutputData.TrialSettings.fsEMG; % [s] Porti EMG device
        % High-pass EMG to remove movement artifacts
        [bHpEMG,aHpEMG] = butter(Settings.nHpEMG,Settings.fcHpEMG/(1/(2*DataEMG.dt)),'high');
        % Select EMG signals & high-pass filter: Sol, TA, GM, GL, Sync (position-based)
        DataEMG.sol = abs(filter(bHpEMG,aHpEMG,detrend(EMGData(Settings.EMGDataRows(1),:)','constant'))); % [\muV] Soleus
        DataEMG.ta  = abs(filter(bHpEMG,aHpEMG,detrend(EMGData(Settings.EMGDataRows(2),:)','constant'))); % [\muV] Tib. Ant.
        DataEMG.gm  = abs(filter(bHpEMG,aHpEMG,detrend(EMGData(Settings.EMGDataRows(3),:)','constant'))); % [\muV] Gastroc. Med.
        DataEMG.gl  = abs(filter(bHpEMG,aHpEMG,detrend(EMGData(Settings.EMGDataRows(4),:)','constant'))); % [\muV] Gastroc. Lat.
        DataEMG.sync = EMGData(Settings.EMGDataRows(5),:)';                                               % [-]    Synchronization signal
    else
        DataEMG = struct(); % Load empty struct, if no EMG processing desired
    end
    
    % Load System Identification data
    data = loadedData.OutputData.SysID;
    % Rough approximate offset between flush and thread
    nOffset = numel(DataMech.t) - numel(data.Reflexive);
    % Time characteristics
    DataSysID.t = (0:numel(DataMech.t)-1)'*DataMech.dt;
    
    % Correct for approximate offset between flush and thread and correct different sign convention
    DataSysID.intrStiffness = [zeros(nOffset,1); -data.Intrinsic(:,1)];
    DataSysID.intrDamping = [zeros(nOffset,1); -data.Intrinsic(:,2)];
    DataSysID.intrInertia = [zeros(nOffset,1); -data.Intrinsic(:,3)];
    DataSysID.reflexive = [zeros(nOffset,1); -data.Reflexive];
    if (isfield(loadedData.OutputData.SysID,'holdEndIndices'))
        DataSysID.holdEndIndices = loadedData.OutputData.SysID.holdEndIndices + nOffset;
        DataSysID.holdStates = loadedData.OutputData.SysID.holdStates;
    end
    % Load Settings
    Settings.pspTStep = loadedData.OutputData.TrialSettings.pspTStep;
    Settings.visFiltSetting = loadedData.OutputData.TrialSettings.visFiltSetting;
end