function [kbi,g,tqVis,States,Settings] = onlineSepStiffIdent(varargin)
%onlineSepStiffIdent Online estimation of Intrinsic stiffness and Reflexive activity
% 	System identification is used to identify a joint stiffness model based 
%	on two parallel paths (also called parallel-cascade model). The model
%	contains an intrinsic path, identifying a 2nd-order rotation spring(K),
%	damper(B), inertia(I) system, and a reflexive path identifying the 
%	gain (G) of the impulse response function IRF.
%
%   The function contains an online implementation of the algorithm such
%   that the estimated parameters are obtained directly. To improve
%   computational effectiveness, the function can take vector inputs
%   corresponding to consecutive sampling times for which the parameters
%   are estimated concurrently.
% Note: See the paper Real-Time Estimation of Intrinsic and Reflex 
% Stiffness by Daniel Ludvig from 2007 for more details about algorithm.
%
%	SYNTAX
%   [~,~,~,States,Settings] = onlineSepStiffIdent(pos,pspTStep,visFiltSetting) {Initialization}
%   [kbi,g,tqLp,States,Settings] = onlineSepStiffIdent(pos,vel,tq,States,Settings) {Execution}
%
%	INPUTS
%   {Initialization}
%	pos           = Perturbation neutral point [rad] (double)
%   pspTStep      = Total step duration [s] (double)
%   visFiltSeting = Visual feedback cut-off frequency [Hz] (double)
%   {Execution}
%	pos      = Position measurement [rad] (Mx1 double)
%   vel      = Velocity measurement [rad/s] (Mx1 double)
%   tq       = Torque measurement [Nm] (Mx1 double)
%   States   = Running states passed between consecutive function calls (struct)
%   Settings = Settings used for every function calls (struct)
%
%   OUTPUT
%   kbi      = Identified intrinsic impedence model [Nm/rad; Nm/rad/s; Nm/rad/s^2] (Mx3 double)
%   g        = Identified reflexive gain [Nm/rad/s] (Mx1 double)
%   tqVis    = Low-pass filtered torque for visualization [Nm] (Mx1 double)
%   States   = Running states passed between consecutive function calls (struct)
%   Settings = Settings used for every function calls (struct)

    %%% Initialization
    if (nargin == 3)
        % Get variables from function input
        pos = varargin{1};
        pspTStep = varargin{2};
        visFiltSetting = varargin{3};
        % Sampling time settings, note that three values are defined as the
        % algorithm contains two downsampling steps
        dt = 1/2048;
        dtDownSample = dt*10;
        dtBuffer = dt*100;
        % Remove all samples before 40ms from reflex IRF estimation
        reflexIRFStart = (floor(0.04/dtDownSample) + 1) + 1; % Additional +1 to compensate Matlab indexing
        % Take reflex IRF length 60ms shorter than PSP step length (equal to Ludvig2007a)
        reflexIRFEnd = round((pspTStep-0.06)/dtDownSample) + 1; % Additional +1 to compensate Matlab indexing
        % Store the position neutral point of the perturbation signal
        pspBias = pos;

        % Create Filters: define order (n); cut-off frequency (fc);
        %  pass-band ripple (pbr); create transfer-function (*Filt);
        % Zero mean filter: High-pass filter with low cut-off frequency
        nZeroMeanFilt = 2; % [-]
        fcZeroMeanFilt = 0.033; % [Hz] Ludvig2007a (Sec. IID)
        [zmFilt.b,zmFilt.a] = butter(nZeroMeanFilt,(fcZeroMeanFilt*2*dt),'high');

        % Correlation filter: Low-pass filter with low cut-off frequency,
        % implictly this functions as averaging filter to obtain zero-lag correlation
        nCorrFilt = 2; % [-] Ludvig2006 (Sec. IIB)
        fcCorrFilt = 0.033; % [Hz] Ludvig2007a
        [corrFilt.b,corrFilt.a] = butter(nCorrFilt,(fcCorrFilt*2*dt),'low');

        % Noise filter: Low-pass filter with high cut-off frequency, to
        % filter for experimental settings with low signal-to-noise ratios
        nLpInFilt = 2; % [-] Ludvig2006 (Sec. IIB)
        fcLpInFilt = 100; % [Hz] Ludvig2007a
        [lpInFilt.b,lpInFilt.a] = butter(nLpInFilt,(fcLpInFilt*2*dt),'low');

        % Anti-Aliasing filter: Low-pass filter with cut-off frequency at 
        % 40% of downsampling frequency, implicitly used for anti-aliasing
        nAAFilt = 8; % [-] Ludvig2007a (Sec. IID)
        fcAAFilt = (0.04/dt); % [Hz] Ludvig2007a (Sec. IID)
        pbrAAFilt = 0.05; % [dB] Ludvig2007a (Sec. IID)
        [aaFilt.b,aaFilt.a] = cheby1(nAAFilt,pbrAAFilt,(fcAAFilt*2*dt),'low');

        % Averaging filter: Low-pass filter with low cut-off frequency
        nAvgFilt = 2; % [-] Ludvig2006 (Sec. IIB)
        fcAvgFilt = 0.033; % [Hz] Ludvig2007a
        [avgFilt.b,avgFilt.a] = butter(nAvgFilt,(fcAvgFilt*2*dtBuffer),'low');
        
        % Derivative filter: FIR filter for numerical backwards difference
        % to differentiate measured signals, 4th-order
        backDiffFilt.b = [25/12 -4 3 -4/3 1/4]/dt;
        backDiffFilt.a = 1;

        % Torque low-pass filter: Low-pass filter use for visualization of
        % biofeedback to subject, not used by system identification
        fcTqLpFilt = visFiltSetting; % [Hz]
        [tqLpFilt.b,tqLpFilt.a] = dZOHLpFilter(fcTqLpFilt*2*pi,1,dt);

        % Allocate all states used within the algorithm in structure
        States.tqVis = zeros(2,1);
        States.posLp = zeros(2,1);
        States.velLp = zeros(2,1);
        States.tqLp = zeros(2,1);
        States.acc = zeros(4,1);
        States.tqHp = zeros(2,1);
        States.posPos = zeros(2,1);
        States.posVel = zeros(2,1);
        States.posAcc = zeros(2,1);
        States.velVel = zeros(2,1);
        States.velAcc = zeros(2,1);
        States.accAcc = zeros(2,1);
        States.tqPos = zeros(2,1);
        States.tqVel = zeros(2,1);
        States.tqAcc = zeros(2,1);
        States.tqIntrHp = zeros(2,1);
        States.velRectHp = zeros(2,1);
        States.tqResReflLp = zeros(8,1);
        States.velRectHpLp = zeros(8,1);
        States.countDownSample = 10;
        States.laggedVelRect = ones(1,reflexIRFEnd);
        States.hankelVelRect = ones(205,reflexIRFEnd - reflexIRFStart + 1);
        States.laggedTqResRefl = ones(205,1);
        States.countBuffer = 100;
        States.gAvg = zeros(2,1);
        States.g = 0;

        % Generate settings to be used by algorithm in structure
        Settings.dt = dt;
        Settings.dtDownSample = dtDownSample;
        Settings.dtBuffer = dtBuffer;
        Settings.reflexIRFStart = reflexIRFStart;
        Settings.reflexIRFEnd = reflexIRFEnd;
        Settings.zmFilt = zmFilt;
        Settings.corrFilt = corrFilt;
        Settings.lpInFilt = lpInFilt;
        Settings.aaFilt = aaFilt;
        Settings.avgFilt = avgFilt;
        Settings.backDiffFilt = backDiffFilt;
        Settings.visFilt = tqLpFilt;
        Settings.pspBias = pspBias;

        % Empty system identification parameters for initialization step
        kbi = [];
        g = [];
        tqVis = [];
        
    %%% Execution
    else
        % Get variables from function input
        pos      = varargin{1};
        vel      = varargin{2};
        tq       = varargin{3};
        States   = varargin{4};
        Settings = varargin{5};
        % Low-pass filter the torque for visualization
        [tqVis,States.tqVis] = filter(Settings.visFilt.b,Settings.visFilt.a,tq,States.tqVis);
        
        %%% 1. Input processing
		% Remove neutral position from signal        
        pos = pos - Settings.pspBias;
		% Low-pass filter inputs to reduce noise
        [posLp,States.posLp] = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,pos,States.posLp);
        [velLp,States.velLp] = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,vel,States.velLp);
        [tqLp,States.tqLp] = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,tq,States.tqLp);

        %%% 2. Acceleration via Velocity Derivative
        [acc,States.acc] = filter(Settings.backDiffFilt.b,Settings.backDiffFilt.a,velLp,States.acc);

        %%% 3. High-pass Torque Filter
        [tqHp,States.tqHp] = filter(Settings.zmFilt.b,Settings.zmFilt.a,tqLp,States.tqHp);

        %%% 4. Correlations
        % Position
        [posPos,States.posPos] = filter(Settings.corrFilt.b,Settings.corrFilt.a,posLp.*posLp,States.posPos);
        [posVel,States.posVel] = filter(Settings.corrFilt.b,Settings.corrFilt.a,posLp.*velLp,States.posVel);
        [posAcc,States.posAcc] = filter(Settings.corrFilt.b,Settings.corrFilt.a,posLp.*acc,States.posAcc);
        % Velocity (vel.*pos already computed above)
        [velVel,States.velVel] = filter(Settings.corrFilt.b,Settings.corrFilt.a,velLp.*velLp,States.velVel);
        [velAcc,States.velAcc] = filter(Settings.corrFilt.b,Settings.corrFilt.a,velLp.*acc,States.velAcc);
        % Acceleration (acc.*pos and acc.*vel already computed above)
        [accAcc,States.accAcc] = filter(Settings.corrFilt.b,Settings.corrFilt.a,acc.*acc,States.accAcc);
        % Torques
        [tqPos,States.tqPos] = filter(Settings.corrFilt.b,Settings.corrFilt.a,tqHp.*posLp,States.tqPos);
        [tqVel,States.tqVel] = filter(Settings.corrFilt.b,Settings.corrFilt.a,tqHp.*velLp,States.tqVel);
        [tqAcc,States.tqAcc] = filter(Settings.corrFilt.b,Settings.corrFilt.a,tqHp.*acc,States.tqAcc);
		
        % Allocate matrices for intrinsic stiffness estimation
        N = numel(pos);
        kbi = zeros(3,N);
        corrMatrix = zeros(3,3);
        corrVect = zeros(3,1);
        for i = 1:N
            % Concatenate Matrix & Vector
            corrMatrix(1) = posPos(i); corrMatrix(4) = posVel(i); corrMatrix(7) = posAcc(i);
            corrMatrix(2) = posVel(i); corrMatrix(5) = velVel(i); corrMatrix(8) = velAcc(i);
            corrMatrix(3) = posAcc(i); corrMatrix(6) = velAcc(i); corrMatrix(9) = accAcc(i);
            corrVect(1) = tqPos(i); corrVect(2) = tqVel(i); corrVect(3) = tqAcc(i);

            %%% 5. Estimate Intrinsic Stiffness (least-squares approximation)
            kbi(:,i) = corrMatrix\corrVect;
        end
        % Remove all not-a-numbers
        kbi(isnan(kbi)) = 0;
        
        %%% 6. Estimate Intrinsic Torque
        posVect = [posLp, velLp, acc];
        % Use matrix with position, velocity and acceleration measurements
        % or estimates and multiply with parameter estimates of K, B and I.
        tqIntr = diag(posVect*kbi);
        [tqIntrHp,States.tqIntrHp] = filter(Settings.zmFilt.b,Settings.zmFilt.a,tqIntr,States.tqIntrHp);
        
		%%% 7. Estimate reflexive torque as residual of high-pass filtered total
        % torque minus estimated intrinsic torque contribution.
        tqResRefl = tqHp - tqIntrHp;
		% Zero-mean Half-wave Rectified Velocity
        velRect = velLp;
        velRect(velRect < 0) = 0;
        [velRectHp,States.velRectHp] = filter(Settings.zmFilt.b,Settings.zmFilt.a,velRect,States.velRectHp);

        %%% 8. Filtering & Downsampling
        % Anti-aliasing filter before downsampling reflexive torque and
        % half-wave rectified velocity
        [tqResReflLp,States.tqResReflLp] = filter(Settings.aaFilt.b,Settings.aaFilt.a,tqResRefl,States.tqResReflLp);
        [velRectHpLp,States.velRectHpLp] = filter(Settings.aaFilt.b,Settings.aaFilt.a,velRectHp,States.velRectHpLp);

        % Allocate vector of reflexive gain estimates
        g = zeros(N,1);
        for i = 1:N
            % Downsample condition based on the relative amount of samples processed
            if States.countDownSample >= (Settings.dtDownSample/Settings.dt)
                % Update sampling counter
                States.countDownSample = States.countDownSample - (Settings.dtDownSample/Settings.dt);
                
                % Build rectified velocity Hankel-matrix
                % Generate lagged velocity vector
                States.laggedVelRect = [velRectHpLp(i), States.laggedVelRect(1:end-1)];
                % Concetenate lagged velocity vectors into Hankel matrix
                States.hankelVelRect = [States.laggedVelRect(Settings.reflexIRFStart:end); States.hankelVelRect(1:end-1,:)];
                % Generate lagged reflexive torque vector
                States.laggedTqResRefl = [tqResReflLp(i); States.laggedTqResRefl(1:end-1)];
            end

            % Downsample condition based on the relative amount of samples processed
            if States.countBuffer >= (Settings.dtBuffer/Settings.dt)
                % Update sampling counter
                States.countBuffer = States.countBuffer - (Settings.dtBuffer/Settings.dt);

                %%% 9. Estimate reflexive IRF
                % Avoid estimation of least-square solution when velocity
                % matrix is not yet full rank
                if (States.hankelVelRect(size(States.hankelVelRect,2),1) ~=1)
                    % Estimate impulse response function from ~40 to 400 ms
                    irfRefl = States.hankelVelRect\States.laggedTqResRefl;
                    %%% 10. Estimate reflexive gain
                    % Gain is estimated by integrating the estimated IRF
                    gainRefl = sum(irfRefl);
                else
                    gainRefl = 0;
                end
                % Smoothen estimated reflexive gain using averaging filter
                [States.g,States.gAvg] = filter(Settings.avgFilt.b,Settings.avgFilt.a,gainRefl,States.gAvg);
            end
            g(i) = States.g;

            %%% Increment Sampling Counters
            States.countDownSample = States.countDownSample + 1;
            States.countBuffer = States.countBuffer + 1;
        end
    end
end