function plotEMGEnsemble(DataMech,DataEMG,EMGEns,Settings)
%plotEMGEnsemble Plot EMG ensemble data
%   Plot all the EMG data, raw and aligned/sorted into ensembles at
%   stretch/perturbation onset.
%
%	SYNTAX
%   plotEMGEnsemble(DataMech,DataEMG,EMGEns,Settings)
%
%	INPUTS
%	DataMech = Selection of (bio)mechanical signals (struct)
%   DataEMG  = Selection of EMG signals (struct)
%   EMGEns   = Aligned and sorted EMG signals (struct)
%   Settings = Overview of all processing settings (struct)

    % EMG Sample timing consistency
    figure('Name','EMG Sample timing consistency');
        hold on;
        diffSawtooth = diff(DataEMG.rawSawtooth);
        isMissingSample = (diffSawtooth ~= 2) & (diffSawtooth ~= -62);
        plot(diffSawtooth);
        plot(find(isMissingSample),diffSawtooth(isMissingSample),'x','LineWidth',2);
        xlabel('Samples [-]')
        ylabel('\Delta Samples [-]')
        title('Missing samples?')
        
    % Check EMG synchronization (start/end)
    figure('Name','Check EMG synchronization (start/end)');
    subplot(2,1,1)
        hold on;
        plot(DataMech.t,DataMech.sync,DataEMG.t,DataEMG.syncEMGScaled)
        legend({'Mechanical','EMG'})
        ylabel('Position [rad]')
        xlim([DataMech.t(1) DataMech.t(1)+7])
    subplot(2,1,2)
        hold on;
        plot(DataMech.t,DataMech.sync,DataEMG.t,DataEMG.syncEMGScaled)
        xlabel('Time [s]')
        ylabel('Position [rad]')
        xlim([DataMech.t(end)-7 DataMech.t(end)])
            
    if (Settings.isPlot > 1)
        % Check correct location of rise/fall locations reference signal
        figure('Name','Check correct location of rise/fall EMG locations reference signal');
            hold on;
            plot(DataEMG.t,DataEMG.syncEMGScaled);
            plot(DataEMG.t(EMGEns.StretchIdxs.rise),DataEMG.syncEMGScaled(EMGEns.StretchIdxs.rise),'x','Color',[0.85 0.325 0.098]);
            plot(DataEMG.t(EMGEns.StretchIdxs.step),DataEMG.syncEMGScaled(EMGEns.StretchIdxs.step),'o','Color',[0.85 0.325 0.098]);
            plot(DataEMG.t(EMGEns.StretchIdxs.pulse),DataEMG.syncEMGScaled(EMGEns.StretchIdxs.pulse),'p','Color',[0.85 0.325 0.098]);
            plot(DataEMG.t(EMGEns.StretchIdxs.fall),DataEMG.syncEMGScaled(EMGEns.StretchIdxs.fall),'x','Color',[0.929 0.694 0.125]);
            plot(DataEMG.t(EMGEns.StretchIdxs.stepFall),DataEMG.syncEMGScaled(EMGEns.StretchIdxs.stepFall),'o','Color',[0.929 0.694 0.125]);
            xlabel('Time [s]')
            ylabel('Position [rad]')
            legend({'Pos.','Rise','Rise-Step','Rise-Pulse','Fall','Fall-Step'})

        % Position/Torque & EMG (SOL/TA) signals
          % Zoom for visibility reflexive torque/EMG
        figure('Name','Position/Torque & EMG (SOL/TA) signals');
            hold on;
            plot(DataMech.t,DataMech.posMeas)
            plot(DataEMG.t,[DataEMG.sol./max(DataEMG.sol)/10,DataEMG.ta./max(DataEMG.ta)/10])
            plot(DataMech.t,(DataMech.tqMeas-DataMech.tqMeas(1))/max(DataMech.tqMeas)/10)
            legend({'Pos.','SOL','TA','Tq.'})
            xlabel('Time [s]')
    end

    % Check for dorsiflexion EMG responses
    figure('Name','Check for dorsiflexion EMG responses');
    subplot(2,2,1)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.rise.solReflex,2))
        title('Soleus')
        ylabel('EMG [\muV]')
    subplot(2,2,2)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.rise.taReflex,2))
        title('Tib. Ant.')
    subplot(2,2,3)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.rise.gmReflex,2))
        plot(EMGEns.tSeg,mean(EMGEns.rise.glReflex,2))
        title('Gastroc. Med./Lat.')
        xlabel('Time [s]')
        ylabel('EMG [\muV]')
    subplot(2,2,4)
        hold on; grid on;
        plot(EMGEns.tSeg,EMGEns.rise.syncReflex)
        title('Sync. Position')
        xlabel('Time [s]')

    % Check for plantarflexion EMG responses
    figure('Name','Check for plantarflexion EMG responses');
    subplot(2,2,1)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.stepFall.solReflex,2))
        title('Soleus')
        ylabel('EMG [\muV]')
    subplot(2,2,2)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.stepFall.taReflex,2))
        title('Tib. Ant.')
    subplot(2,2,3)
        hold on; grid on;
        plot(EMGEns.tSeg,mean(EMGEns.stepFall.gmReflex,2))
        plot(EMGEns.tSeg,mean(EMGEns.stepFall.glReflex,2))
        title('Gastroc. Med./Lat.')
        xlabel('Time [s]')
        ylabel('EMG [\muV]')
    subplot(2,2,4)
        hold on; grid on;
        plot(EMGEns.tSeg,EMGEns.stepFall.syncReflex)
        title('Sync. Position')
        xlabel('Time [s]')

    if (Settings.isPlot > 1)
        % Check for Soleus post-Step/-Pulse similarity
        figure('Name','Check for Soleus post-Step/-Pulse similarity');
            plot(EMGEns.tSeg,[mean(EMGEns.postStep.solReflex,2) mean(EMGEns.postPulse.solReflex,2)])
            legend({'post-Step','post-Pulse'})
            xlabel('Time [s]')
            ylabel('EMG [\muV]')
            title('Similar post-Step & -Pulse reflex?')
    end
end