function plotMechEnsemble(DataMech,MechEns,Settings)
%plotMechEnsemble Plot (bio)mechanical ensemble data
%   Plot all the (bio)mechanical data, raw and aligned/sorted into 
%   ensembles at stretch/perturbation onset.
%
%	SYNTAX
%   plotMechEnsemble(DataMech,MechEns,Settings)
%
%	INPUTS
%	DataMech = Selection of (bio)mechanical signals (struct)
%   MechEns  = Aligned and sorted (bio)mechanical signals (struct)
%   Settings = Overview of all processing settings (struct)

    % Achilles sample timing consistency
    figure('Name','Achilles sample timing consistency');
        hold on;
        plot(diff(DataMech.t));
        xlabel('Time [s]')
        ylabel('\DeltaTime [s]')
        
    % Position/Torque references and signals
      % Zoom for perturbation execution; Visibility reflexive torque
    figure('Name','Position/Torque references and signals');
        hold on;
        yyaxis left;
        plot(DataMech.t,[DataMech.posRef,DataMech.posMeas]);
        xlabel('Time [s]')
        ylabel('Position [rad]')
        yyaxis right;
        plot(DataMech.t,DataMech.tqMeas);
        ylabel('Torque [Nm]')
        
    if (Settings.isPlot > 1)
        % Check correct location of rise/fall locations reference signal
        figure('Name','Check correct location of rise/fall locations reference signal');
            hold on;
            plot(DataMech.t,DataMech.posRef);
            plot(DataMech.t(MechEns.StretchIdxs.rise),DataMech.posRef(MechEns.StretchIdxs.rise),'x','Color',[0.85 0.325 0.098]);
            plot(DataMech.t(MechEns.StretchIdxs.step),DataMech.posRef(MechEns.StretchIdxs.step),'o','Color',[0.85 0.325 0.098]);
            plot(DataMech.t(MechEns.StretchIdxs.pulse),DataMech.posRef(MechEns.StretchIdxs.pulse),'p','Color',[0.85 0.325 0.098]);
            plot(DataMech.t(MechEns.StretchIdxs.fall),DataMech.posRef(MechEns.StretchIdxs.fall),'x','Color',[0.929 0.694 0.125]);
            plot(DataMech.t(MechEns.StretchIdxs.stepFall),DataMech.posRef(MechEns.StretchIdxs.stepFall),'o','Color',[0.929 0.694 0.125]);
            xlabel('Time [s]')
            ylabel('Position [rad]')
            legend({'Pos.','Rise','Rise-Step','Rise-Pulse','Fall','Fall-Step'})

        % Ensemble position/torque responses
        figure('Name','Ensemble position/torque responses');
        subplot(2,1,1)
            hold on, box off
            h1 = plot(MechEns.tSeg,MechEns.rise.posRef,'k--');
            plot(MechEns.tSeg,MechEns.fall.posRef,'k--');
            h2 = plot(MechEns.tSeg,MechEns.rise.pos,'Color',[0 0.447 0.741]);
            h3 = plot(MechEns.tSeg,MechEns.fall.pos,'Color',[0.85 0.325 0.098]);
            xlim([-0.02 0.08])
            ylabel('Position [rad]')
            legend([h1(1),h2(1),h3(1)],{'Ref.','Rise','Fall'})
        subplot(2,1,2)
            hold on, box off
            plot(MechEns.tSeg,MechEns.rise.tq,'Color',[0 0.447 0.741]);
            plot(MechEns.tSeg,MechEns.fall.tq,'Color',[0.85 0.325 0.098]);
            xlim([-0.02 0.08])
            xlabel('Time [s]')
            ylabel('Torque [Nm]')

        % Ensemble averaged position/torque responses
        figure('Name','Ensemble averaged position/torque responses');
        % Position
        subplot(2,1,1)
            hold on, box off
            h1 = plot(MechEns.tSeg,MechEns.rise.posRef,'k--');
            plot(MechEns.tSeg,MechEns.fall.posRef,'k--');
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.pos,2)-std(MechEns.rise.pos,[],2);flipud(mean(MechEns.rise.pos,2)+std(MechEns.rise.pos,[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            h2 = plot(MechEns.tSeg,mean(MechEns.rise.pos,2),'Color',[0 0.447 0.741]);
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.fall.pos,2)-std(MechEns.fall.pos,[],2);flipud(mean(MechEns.fall.pos,2)+std(MechEns.fall.pos,[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            h3 = plot(MechEns.tSeg,mean(MechEns.fall.pos,2),'Color',[0.85 0.325 0.098]);
            xlim([-0.02 0.04])
            ylabel('Position [rad]')
            legend([h1(1),h2(1),h3(1)],{'Ref.','Rise','Fall'})
        % Torque
        subplot(2,1,2)
            hold on, box off
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.tqComp,2)-std(MechEns.rise.tqComp,[],2);flipud(mean(MechEns.rise.tqComp,2)+std(MechEns.rise.tqComp,[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.rise.tqComp,2),'Color',[0 0.447 0.741])
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.fall.tqComp,2)-std(MechEns.fall.tqComp,[],2);flipud(mean(MechEns.fall.tqComp,2)+std(MechEns.fall.tqComp,[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.fall.tqComp,2),'Color',[0.85 0.325 0.098])
            xlim([-0.02 0.04])
            xlabel('Time [s]')
            ylabel('Torque [Nm]')

        % Ensemble averaged position/torque responses from pulses/steps
        figure('Name','Ensemble averaged position/torque responses from pulses/steps');
        % Position
        subplot(2,1,1)
            hold on, box off
            h1 = plot(MechEns.tSeg,MechEns.rise.posRef(:,MechEns.StretchIdxs.nStep),'k--');
            plot(MechEns.tSeg,MechEns.rise.posRef(:,MechEns.StretchIdxs.nPulse),'k--');
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep),2)-std(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep),[],2);flipud(mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep),2)+std(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            h2 = plot(MechEns.tSeg,mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep),2),'Color',[0 0.447 0.741]);
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),2)-std(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),[],2);flipud(mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),2)+std(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            h3 = plot(MechEns.tSeg,mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),2),'Color',[0.85 0.325 0.098]);
            xlim([-0.02 0.4])
            ylabel('Position [rad]')
            legend([h1(1),h2(1),h3(1)],{'Ref.','Step','Pulse'})
        % Torque
        subplot(2,1,2)
            hold on, box off
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep),2)-std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep),[],2);flipud(mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep),2)+std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep),2),'Color',[0 0.447 0.741])
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse),2)-std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse),[],2);flipud(mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse),2)+std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse),[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse),2),'Color',[0.85 0.325 0.098])
            xlim([-0.02 0.4])
            xlabel('Time [s]')
            ylabel('Torque [Nm]')
            
        % Ensemble averaged position/torque responses from plantarflexion steps
        figure('Name','Ensemble averaged position/torque responses from plantarflexion steps');
        % Position
        subplot(2,1,1)
            hold on, box off
            h1 = plot(MechEns.tSeg,MechEns.fall.posRef(:,MechEns.StretchIdxs.nStepFall),'k--');
            plot(MechEns.tSeg,MechEns.fall.posRef(:,MechEns.StretchIdxs.nStepFall),'k--');
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.fall.pos(:,MechEns.StretchIdxs.nStepFall),2)-std(MechEns.fall.pos(:,MechEns.StretchIdxs.nStepFall),[],2);flipud(mean(MechEns.fall.pos(:,MechEns.StretchIdxs.nStepFall),2)+std(MechEns.fall.pos(:,MechEns.StretchIdxs.nStepFall),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            h2 = plot(MechEns.tSeg,mean(MechEns.fall.pos(:,MechEns.StretchIdxs.nStepFall),2),'Color',[0 0.447 0.741]);
            xlim([-0.02 0.4])
            ylabel('Position [rad]')
            legend([h1(1),h2(1)],{'Ref.','Step'})
        % Torque
        subplot(2,1,2)
            hold on, box off
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.fall.tqComp(:,MechEns.StretchIdxs.nStepFall),2)-std(MechEns.fall.tqComp(:,MechEns.StretchIdxs.nStepFall),[],2);flipud(mean(MechEns.fall.tqComp(:,MechEns.StretchIdxs.nStepFall),2)+std(MechEns.fall.tqComp(:,MechEns.StretchIdxs.nStepFall),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.fall.tqComp(:,MechEns.StretchIdxs.nStepFall),2),'Color',[0 0.447 0.741])
            xlim([-0.02 0.4])
            xlabel('Time [s]')
            ylabel('Torque [Nm]')
            
        % Ensemble averaged position/torque responses from post-pulses/steps
        figure('Name','Ensemble averaged position/torque responses from post-pulses/steps');
        % Position
        subplot(2,1,1)
            hold on, box off
            h1 = plot(MechEns.tSeg,MechEns.rise.posRef(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),'k--');
            plot(MechEns.tSeg,MechEns.rise.posRef(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),'k--');
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2)-std(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),[],2);flipud(mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2)+std(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            h2 = plot(MechEns.tSeg,mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2),'Color',[0 0.447 0.741]);
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2)-std(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),[],2);flipud(mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2)+std(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            h3 = plot(MechEns.tSeg,mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2),'Color',[0.85 0.325 0.098]);
            xlim([-0.3 0.2])
            ylabel('Position [rad]')
            legend([h1(1),h2(1),h3(1)],{'Ref.','Step','Pulse'})
        % Torque
        subplot(2,1,2)
            hold on, box off
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2)-std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),[],2);flipud(mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2)+std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),[],2))],[0 0.447 0.741],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nStep(MechEns.StretchIdxs.nStep ~= numel(MechEns.StretchIdxs.rise))+1),2),'Color',[0 0.447 0.741])
            fill([MechEns.tSeg;flipud(MechEns.tSeg)],[mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2)-std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),[],2);flipud(mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2)+std(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),[],2))],[0.85 0.325 0.098],'EdgeColor','none','FaceAlpha',0.5)
            plot(MechEns.tSeg,mean(MechEns.rise.tqComp(:,MechEns.StretchIdxs.nPulse(MechEns.StretchIdxs.nPulse ~= numel(MechEns.StretchIdxs.rise))+1),2),'Color',[0.85 0.325 0.098])
            xlim([-0.3 0.2])
            xlabel('Time [s]')
            ylabel('Torque [Nm]')
    end

    % Ensemble averaged position/velocity/acceleration/torque responses from pulses
    figure('Name','Ensemble averaged position/velocity/acceleration/torque responses from pulses');
    subplot(2,2,1)
        hold on; grid on;
        plot(MechEns.tSeg,mean(MechEns.rise.pos(:,MechEns.StretchIdxs.nPulse),2)); 
        ylabel('Position [rad]')
        xlim([-0.02 0.25])
    subplot(2,2,2)
        hold on; grid on;
        plot(MechEns.tSeg,mean(MechEns.rise.vel(:,MechEns.StretchIdxs.nPulse),2));
        ylabel('Velocity [rad/s]')
        xlim([-0.02 0.25])
    subplot(2,2,3)
        hold on; grid on;
        plot(MechEns.tSeg,mean(MechEns.rise.acc(:,MechEns.StretchIdxs.nPulse),2)); 
        xlabel('Time [s]')
        ylabel('Acceleration [rad/s^2]')
        xlim([-0.02 0.25])
    subplot(2,2,4)
        hold on; grid on;
        plot(MechEns.tSeg,mean(MechEns.rise.tq(:,MechEns.StretchIdxs.nPulse),2));
        xlabel('Time [s]')
        ylabel('Torque [Nm]')
        xlim([-0.02 0.25])
end