function [tEMG,syncEMGScaled] = synchronizePosEMG(syncMech,syncEMG,tMech,dtMech,dtEMG)
%synchronizePosEMG Synchronize EMG and (bio)mechanical data
%   Synchronizes EMG and mechanical measurements based on either a position
%   or motor on/off synchronization signal measured twice: using the 
%   robotic system and measured as analog or digital trigger signal via the
%   EMG system.
%
%	SYNTAX
%   [tEMG,syncEMGScaled] = synchronizePosEMG(syncMech,syncEMG,tMech,dtMech,dtEMG)
%
%	INPUTS
%   syncMech = Measured position at motor encoder or motor on/off signal (Mx1 double)
%   syncEMG  = Analog or Digital trigger measurement of synchMech with EMG (Nx1 double)
%   tMech    = Time vector of mechanical data (Mx1 double)
%   dtMech   = Sampling time mechanical data [s] (double)
%   dtEMG    = Sampling time EMG [s] (double)
%
%   OUTPUTS
%   tEMG          = Synchronized time vector of EMG (Nx1 double)
%   syncEMGScaled = EMG synchronization rescaled to reference (Nx1 double)
    
    % Generate histogram of synchronization signal, to retrieve signal minimum/maximum (without overshoots)
    [N,X] = hist(syncMech,5000);
    [~,analogLowMech] = max(N(1:floor(3*end/4))); analogLowMech = X(analogLowMech);
    [~,analogHighMech] = max(N(ceil(3*end/4):end)); analogHighMech = X(analogHighMech+ceil(3*numel(X)/4)-1);
    [N,X] = hist(syncEMG,5000);
    [~,analogLowEMG] = max(N(1:floor(3*end/4))); analogLowEMG = X(analogLowEMG);
    [~,analogHighEMG] = max(N(ceil(3*end/4):end)); analogHighEMG = X(analogHighEMG+ceil(3*numel(X)/4)-1);
    % Scale EMG sync. signal to position vector
    syncEMGScaled = (syncEMG-analogLowEMG)/(analogHighEMG-analogLowEMG)*(analogHighMech-analogLowMech) + analogLowMech;
    if (all(mod(syncMech,1) == 0)) 
        syncEMGScaled = round(syncEMGScaled); % Make EMG integer, if mech sync was integer
    end
    % Resample EMG signal to motor sampling frequency, if unequal
    if (dtMech == dtEMG)
        syncEMGScaledRes = syncEMGScaled;
    elseif (dtMech > dtEMG)
        syncEMGScaledRes = syncEMGScaled(1:(dtMech/dtEMG):end);
    else
        syncEMGScaledRes = reshape(repmat(syncEMGScaled,1,dtEMG/dtMech)',[],1);
    end
    
    % Retrieve start and end of position & EMG signals; Co-locate first and
    % last EMG Trigger ON sample or perturbation midpoint.
    idxEMGStart = find(syncMech > 0,1,'first') - find(syncEMGScaledRes > 0,1,'first');
    if (syncEMGScaledRes(end) <= 0) % EMG finishes low, search falling edge
        if (syncMech(end) <= 0) % Mech also finishes low, direct falling edge search
            idxEMGEnd = find(flipud(syncEMGScaledRes) > 0,1,'first') - find(flipud(syncMech) > 0,1,'first');
        else % Mech finishes high, skip to first low zone before falling edge search
            idxEMGEnd = find(flipud(syncEMGScaledRes) > 0,1,'first') - (find(flipud(syncMech(1:find(syncMech <= 0,1,'last'))) > 0,1,'first') + numel(syncMech) - find(syncMech <= 0,1,'last'));
        end
    else % EMG finishes high, search rising edge
        if (syncMech(end) <= 0) % Mech finishes low, skip to first high zone before rising edge search
            idxEMGEnd = find(flipud(syncEMGScaledRes) <= 0,1,'first') - (find(flipud(syncMech(1:find(syncMech > 0,1,'last'))) <= 0,1,'first') + numel(syncMech) - find(syncMech > 0,1,'last'));
        else % Mech also finishes low, direct rising edge search
            idxEMGEnd = find(flipud(syncEMGScaledRes) <= 0,1,'first') - find(flipud(syncMech) <= 0,1,'first');
        end
    end
    % Get starting time (from vector or extrapolation)
    tEMGStart = tMech(1+max(idxEMGStart,0));
    % Get end time (from vector or extrapolation)
    tEMGEnd = tMech(end+min(idxEMGEnd,0));
    % Generate the EMG time vector from start to end, matching number of samples to sync signal
    tEMGRes = linspace(tEMGStart,tEMGEnd,numel(syncEMGScaledRes)+min(idxEMGStart,0)-max(idxEMGEnd,0))';
    tEMGRes = [(idxEMGStart:-1)'*mean(diff(tEMGRes))+tEMGRes(1); tEMGRes; (1:idxEMGEnd)'*mean(diff(tEMGRes))+tEMGRes(end)];
    % Resample EMG signal back to original EMG samling frequency, if unequal
    if (dtMech == dtEMG)
        tEMG = tEMGRes;
    elseif (dtMech > dtEMG)
        tEMG = tEMGRes(1) + (0:numel(syncEMGScaled)-1)'*mean(diff(tEMGRes))/2;
    else
        tEMG = tEMGRes(1:(dtEMG/dtMech):end);
    end
end