function [error,VAFPerc,tqsEns] = vafSimulation(paramWZ,holdIdx,DataMech,DataSysID,DataFilt,MechEns)
%vafSimulation Compute VAF via forward-model simulation
%   Computed VAF using the measured and modeled torque output of the PC
%   model. Modeled torque computed via forward-model simulation. The
%   simulation inputs (DataFilt) are equivalent in terms of high/low-pass
%   filtering and delay as online PC algorithm. Function input/outputs are
%   ordered such that function can be used within post-trial
%   nonlinear-least squares optimization to estimate activation dynamics.
%
%	SYNTAX
%   [error,VAFPerc,tqsEns] = vafSimulation(paramWZ,holdIdx,DataMech,DataSysID,DataFilt,MechEns)
%
%	INPUTS
%   paramWZ   = Activation dynamics parameters [\omega \zeta] (2x1 double)
%   holdIdx   = Hold period index within trial (double)
%   DataMech  = Selection of (bio)mechanical signals (struct)
%   DataSysID = System Identification (seperate intrinsic/reflexive) results (struct)
%   DataFilt  = Processed (bio)mechanical signals, inputs to online PC algorithm (struct)
%   MechEns   = Aligned and sorted (bio)mechanical signals (struct)
%
%   OUTPUTS
%	error   = Residual ensemble torque (Modeled - Measured) (Mx1 double)
%   VAFPerc = %VAF computed via modeled and measured torque output (double)
%   tqsEns  = Modeled and measured torque outputs (Mx2 double)

    % Model parameters
    w = paramWZ(1);
    z = paramWZ(2);
    Hact = c2d(tf(w^2,[1 2*w*z w^2]),DataMech.dt,'tustin');
    
    % Model Torque
    tqIntr = DataFilt.acc.*DataSysID.intrInertia + DataFilt.vel.*DataSysID.intrDamping + DataFilt.pos.*DataSysID.intrStiffness;
    tqRefl = DataSysID.reflexive.*lsim(Hact,DataFilt.velRectDelay,[]);
    tqSim = -tqIntr-tqRefl;
    
    % Ensemble Averages
    pertIdxs = MechEns.StretchIdxs.rise(MechEns.StretchIdxs.rise > (DataSysID.holdEndIndices(holdIdx)-60*2048+1) & MechEns.StretchIdxs.rise < DataSysID.holdEndIndices(holdIdx)) - (DataSysID.holdEndIndices(holdIdx)-60*2048);
    tqHold = DataFilt.tq((DataSysID.holdEndIndices(holdIdx)-60*2048+1):DataSysID.holdEndIndices(holdIdx));
    tqSimHold = tqSim((DataSysID.holdEndIndices(holdIdx)-60*2048+1):DataSysID.holdEndIndices(holdIdx));
    
    % Ensemble torques
    [tqHoldEns] = alignTqEns(tqHold,pertIdxs);
    [tqSimHoldEns] = alignTqEns(tqSimHold,pertIdxs);
    tqsEns = [tqHoldEns,tqSimHoldEns];
    
    % Nonlinear Least-Squares Optimalization output
    error = tqSimHoldEns-tqHoldEns;
    VAFPerc = (1 - var(error)/var(tqHoldEns))*100;
end

function [tqEns] = alignTqEns(tq,pertIdxs)
%alignTqEns Retrieve torque response ensembles to stretches
%   Retrieve all the seperate torque response ensembles to all stretches
%   starting at the provided mechanical data indices.
%
%	SYNTAX
%   [tqEns] = alignTqEns(tq,pertIdxs)
%
%	INPUTS
%   tq       = Measured/Modeled torque vector (Mx1 double)
%   pertIdxs = Set of start indices for stretch reflexes in mechanial data (Nx1 double)
%
%   OUTPUT
%   tqEns = Aligned and sorted torque signal (struct)

    dt = 1/2048; % Achilles Sampling time [s]
    % Get background and response index vectors from perturbation indices,
    % similar to 'getMechStretchEnsembles'
    backIdxs = arrayfun(@(x) (x-round(0.04/dt)):x-1,pertIdxs,'UniformOutput',false);
    ensembleIdxs = arrayfun(@(x,y) x:(x+round(0.4/dt)-1),pertIdxs(1:end-1),'UniformOutput',false);
    % Remove first/last stretch if not enough data is available
    % before/after the stretch onset
    if (any(backIdxs{1} <= 0)), backIdxs(1) = []; ensembleIdxs(1) = []; end
    if (pertIdxs(end) < (60*2048 - round(0.4/dt)))
        ensembleIdxs{end+1} = pertIdxs(end):(pertIdxs(end)+round(0.4/dt)-1); 
    else
        backIdxs(end) = [];
    end
    % Compute background torque as mean before stretch onset
    backTq = cellfun(@(x) mean(tq(x)),backIdxs,'UniformOutput',false);
    % Retrieve torque response ensemble (normalized with background torque)
    tqEns = cellfun(@(x,y) tq(x) - y,ensembleIdxs,backTq,'UniformOutput',false);
    tqEns = mean(cat(2,tqEns{:}),2);
end