%% Clean Start
clc, clearvars, close all;

%% Settings
% Overview of conditions to-be processed
Settings.isEMG = 1; % Flag to process EMG (if true)
Settings.subjectID = cellstr(num2str((1:10).', '%03d')); % Subject identifier (padded to 3 numbers)
% Define data storage location
dataCD = cd; fileSepIdxs = strfind(dataCD,filesep);
dataCD = dataCD(1:fileSepIdxs(end)-1);
% Data column/row order settings
Settings.mechDataColumns = [1 6 7 2 3 4 2]; % [-] Columns used to load data [time, posRef, velRef, posMeas, velMeas, tqMeas, Sync]
Settings.EMGDataRows = [1 2 3 4 5 6]; % [-] Rows used to load data [SOL, TA, GM, GL, Sync, Sawtooth]

% Perturbation
Settings.pertAmpl = 0.035; % [rad] Amplitude (peak-to-peak)
% Filter settings
Settings.fcLpPos = 200;  % [Hz] Low-pass filter position for visualization
Settings.nHpEMG  = 2;    % [-]  High-pass filter order for EMG
Settings.fcHpEMG = 5;    % [Hz] High-pass filter frequency for EMG
% Length of (bio)mechanical data segments
Settings.tSegStart = 0.04; % [s] Time before zero/onset
Settings.tSegEnd   = 0.4; % [s] Time after zero/onset
% Length of EMG data segments
Settings.tSegStartEMG = 0.04; % [s] Time before zero/onset
Settings.tSegEndEMG   = 0.1; % [s] Time after zero/onset
% Create cell array of all perturbation characteristics to be sorted for
Settings.sortFields = {'rise','stepFall'};
% Add folder with all shared functions
addpath(genpath('Functions'));

%% Load & Process Data
% Load EMG windows
load([cd '/Processed/EMGWindows.mat']);
fieldNamesIntr = {'tqTarget','intrK','solBack','taBack','gmBack','glBack'};
fieldNamesRefl = {'tqTarget','reflG','solRefl','taRefl','gmRefl','glRefl'};
for ii = 1:numel(fieldNamesIntr)
    DataIntr.(fieldNamesIntr{ii}) = zeros(20,numel(Settings.subjectID));
    DataRefl.(fieldNamesRefl{ii}) = zeros(20,numel(Settings.subjectID));
end

%% Stretch Analysis
for ii = numel(Settings.subjectID):-1:1
    % Retrieve all experimental folders/files
    dirContent = dir([dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/*.mat']);
    dirContent = {dirContent.name}';
    ll = 1;

    for jj = 1:numel(dirContent)
        dataSet = [dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/' dirContent{jj}];
        % Load (bio)mechanical and [optionally] EMG data
        [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings);
        % Synchronize EMG and motor data based on position sync.
        [DataEMG.t,DataEMG.syncEMGScaled] = synchronizePosEMG(DataMech.sync,DataEMG.sync,DataMech.t,DataMech.dt,DataEMG.dt);
        
        % Align and sort the (bio)mechanical data
        MechEns = alignSortMechData(DataMech,Settings);
        % Align and sort the EMG data
        EMGEns = alignSortEMGData(DataMech,DataEMG,MechEns.StretchIdxs,Settings);
        
        % Intrinsic Results [tq, K, backEMG(4x)]
        DataIntr.tqTarget(ll:ll-1+numel(DataSysID.holdEndIndices),ii) = DataSysID.holdStates(:,1);
        DataIntr.intrK(ll:ll-1+numel(DataSysID.holdEndIndices),ii) = DataSysID.holdStates(:,2);
        % Reflexive Results [tq, G, reflexEMG(4x)]
        DataRefl.tqTarget(ll:ll-1+numel(DataSysID.holdEndIndices),ii) = DataSysID.holdStates(:,1);
        DataRefl.reflG(ll:ll-1+numel(DataSysID.holdEndIndices),ii) = DataSysID.holdStates(:,3);
        for kk = 1:size(DataSysID.holdStates,1)
            % Rise indices for hold-state
            holdIdxs = find(MechEns.StretchIdxs.rise > (DataSysID.holdEndIndices(kk)-60*2048+1) & MechEns.StretchIdxs.rise < DataSysID.holdEndIndices(kk));
            holdIdxsFall = find(MechEns.StretchIdxs.stepFall > (DataSysID.holdEndIndices(kk)-60*2048+1) & MechEns.StretchIdxs.stepFall < DataSysID.holdEndIndices(kk));
            % Background EMG (Sol, TA, GM, GL): mean(background)
            DataIntr.solBack(ll,ii) = mean(mean((EMGEns.rise.solBack(:,holdIdxs))));
            DataIntr.taBack(ll,ii) = mean(mean((EMGEns.rise.taBack(:,holdIdxs))));
            DataIntr.gmBack(ll,ii) = mean(mean((EMGEns.rise.gmBack(:,holdIdxs))));
            DataIntr.glBack(ll,ii) = mean(mean((EMGEns.rise.glBack(:,holdIdxs))));
            % Reflex EMG (Sol, TA, GM, GL): RMS(Half-wave-rectification(Reflex response - mean(background)))
            DataRefl.solRefl(ll,ii) = mean(rms(max(EMGEns.rise.solReflex((EMGWindows(ii).sol(1):EMGWindows(ii).sol(2))+1,holdIdxs) - mean(EMGEns.rise.solBack(:,holdIdxs)),0)));
            DataRefl.taRefl(ll,ii) = mean(rms(max(EMGEns.stepFall.taReflex((EMGWindows(ii).sol(1):EMGWindows(ii).sol(2))+1,holdIdxsFall) - mean(EMGEns.stepFall.taBack(:,holdIdxsFall)),0)));
            DataRefl.gmRefl(ll,ii) = mean(rms(max(EMGEns.rise.gmReflex((EMGWindows(ii).ga(1):EMGWindows(ii).ga(2))+1,holdIdxs) - mean(EMGEns.rise.gmBack(:,holdIdxs)),0)));
            DataRefl.glRefl(ll,ii) = mean(rms(max(EMGEns.rise.glReflex((EMGWindows(ii).ga(1):EMGWindows(ii).ga(2))+1,holdIdxs) - mean(EMGEns.rise.glBack(:,holdIdxs)),0)));
            ll = ll + 1;
        end
    end
end

%%% Remove EMG artifacts and Incorrect executions
DataIntr.gmBack(9:14,4) = NaN; DataRefl.gmRefl(9:14,4) = NaN; % EMG artifacts S004_T4C3R1 GM muscle
DataIntr.taBack(11:15,7) = NaN; DataRefl.taRefl(11:15,7) = NaN; % EMG artifacts S007_T4C3R2 TA muscle
DataIntr.taBack(6:10,8) = NaN; DataRefl.taRefl(6:10,8) = NaN; % EMG artifacts S008_T4C2R1 TA muscle
for ii = 2:numel(fieldNamesIntr)
    DataIntr.(fieldNamesIntr{ii})(12,9) = NaN; DataRefl.(fieldNamesRefl{ii})(12,9) = NaN; % Incorrect task execution
end

%% Correlation (via R)
% Correlations, normalized per torque
for targetTorque = [0 5]
    if (targetTorque == 0)
        isTargetTq = DataIntr.tqTarget > -2.5;
    else
        isTargetTq = DataIntr.tqTarget < -2.5;
    end
    
    dataIntrTq = [DataIntr.solBack(isTargetTq),...
                  DataIntr.taBack(isTargetTq),...
                  DataIntr.gmBack(isTargetTq),...
                  DataIntr.glBack(isTargetTq),...
                  DataIntr.intrK(isTargetTq)];
    dataReflTq = [DataRefl.solRefl(isTargetTq),...
                  DataRefl.taRefl(isTargetTq),...
                  DataRefl.gmRefl(isTargetTq),...
                  DataRefl.glRefl(isTargetTq),...
                  DataRefl.reflG(isTargetTq)];
%     save([cd '/Processed/data' num2str(targetTorque) 'Nm.mat'],'dataIntrTq','dataReflTq'); 
end

%% T-test on torque variability
load([cd '/Processed/dataVAF.mat'])
% Retrieve torque variability from VAF-analysis
DataVAF.lpTqVar = sqrt(DataVAF.lpTqVar);
isTargetTq = reshape(DataIntr.tqTarget > -2.5,[],10); % Get data split between 0 and -5Nm torque targets
% Compute standard deviation per subject 
stdTargetTq0 = sum(DataVAF.lpTqVar.*isTargetTq,'omitnan')./sum(isTargetTq,'omitnan');
isTargetTq(12,9) = true; % Remove incorrect task execution
stdTargetTq5 = sum(DataVAF.lpTqVar.*(~isTargetTq),'omitnan')./sum(~isTargetTq,'omitnan');
% Test the 'within-subject standard deviations' across-subject
disp([num2str(mean(stdTargetTq0),3) ' \pm ' num2str(std(stdTargetTq0),3)])
disp([num2str(mean(stdTargetTq5),3) ' \pm ' num2str(std(stdTargetTq5),3)])
[h,p,ci,stats] = ttest(stdTargetTq0,stdTargetTq5);

%% Figure 3B (Normalized Correlations)
% Show critical methodology of normalizing correlation results
isTargetTq = DataIntr.tqTarget > -2.5;
dataX = reshape(DataIntr.('intrK')(isTargetTq),[],numel(Settings.subjectID));
dataY = reshape(DataIntr.('solBack')(isTargetTq),[],numel(Settings.subjectID));
dataZX = zscorenan(dataX);
dataZY = zscorenan(dataY);
dataIdxs = ~isnan(dataY);

fitSubj = zeros(numel(Settings.subjectID),2);
fitZSubj = zeros(numel(Settings.subjectID),2);
% Linear least-squares method for 1st-order regression fit
for ii = 1:numel(Settings.subjectID)
    % Per-subject (raw & normalized)
    fitSubj(ii,2) = tls(detrend(dataX(dataIdxs(:,ii),ii),'constant'),detrend(dataY(dataIdxs(:,ii),ii),'constant'));
    fitSubj(ii,1) = mean(dataY(dataIdxs(:,ii),ii)) - fitSubj(ii,2)*mean(dataX(dataIdxs(:,ii),ii));
    fitZSubj(ii,2) = tls(detrend(dataZX(dataIdxs(:,ii),ii),'constant'),detrend(dataZY(dataIdxs(:,ii),ii),'constant'));
    fitZSubj(ii,1) = mean(dataZY(dataIdxs(:,ii),ii)) - fitZSubj(ii,2)*mean(dataZX(dataIdxs(:,ii),ii));
end
% Combined (raw & normalized)
fitAll(1,2) = tls(detrend(dataX(dataIdxs),'constant'),detrend(dataY(dataIdxs),'constant'));
fitAll(1,1) = mean(dataY(dataIdxs)) - fitAll(1,2)*mean(dataX(dataIdxs));
fitZAll(1,2) = tls(detrend(dataZX(dataIdxs),'constant'),detrend(dataZY(dataIdxs),'constant'));
fitZAll(1,1) = mean(dataZY(dataIdxs)) - fitZAll(1,2)*mean(dataZX(dataIdxs));

% Settings
plotWidth    = 9.6;
plotHeight   = 5;
marginLeft   = 0.994;
marginRight  = 1.248;
marginBottom = 0.906;
marginTop    = 0.175;
nPlotX       = 2;
nPlotY       = 1;
spacingX     = 1.192;
spacingY     = 0.67;
fontSize     = 8;
fontName     = 'Segoe UI';

% Generation of Figure 3B
figure('Units','centimeters','Position',[4 4 plotWidth plotHeight],'Color','w');
posSubplots = subplotPositions(plotWidth,plotHeight,marginLeft,marginRight,marginBottom,marginTop,nPlotX,nPlotY,spacingX,spacingY);
hAx(1) = axes('position',posSubplots{1},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    xLimits = [0 ceil(max(dataX(dataIdxs))/10)*10]; 
    yLimits = [0 ceil(max(dataY(dataIdxs))/10)*10];
    for ii = numel(Settings.subjectID):-1:1
        plot(0:xLimits(2),fitSubj(ii,1)+(0:xLimits(2))*fitSubj(ii,2),'Color',plotColors(ii,'light'),'LineWidth',1,'HandleVisibility','off');
    end
    plot(0:xLimits(2),fitAll(1)+(0:xLimits(2))*fitAll(2),'k-.','LineWidth',1);
    lineStyles = {'o','x','+','*','s','d','v','>','p','h'};
    for ii = numel(Settings.subjectID):-1:1
        plot(dataX(dataIdxs(:,ii),ii),dataY(dataIdxs(:,ii),ii),'LineStyle','none','Marker',lineStyles{ii},'Color',plotColors(ii),'MarkerSize',7,'LineWidth',1);
    end
    axis([xLimits, yLimits],'square');
    xlabel('Intrinsic Stiffness (Nm/rad)')
    ylabel('SOL Back. EMG (xV)')
    set(gca,'TickLength',[0.02 0.025]);
% Generation of Figure 3C
hAx(2) = axes('position',posSubplots{2},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    xLimits = [-3 3];
    yLimits = [-3 3];
    plot(xLimits,[0 0],'--','Color',plotColors(10,'light')); plot([0 0],yLimits,'--','Color',plotColors(10,'light'));
    for ii = numel(Settings.subjectID):-1:1
        plot(xLimits(1):xLimits(2),fitZSubj(ii,1)+(xLimits(1):xLimits(2))*fitZSubj(ii,2),'Color',plotColors(ii,'light'),'LineWidth',1,'HandleVisibility','off');
    end
    hAx1Plots(numel(Settings.subjectID)+1) = plot(xLimits(1):xLimits(2),fitZAll(1)+(xLimits(1):xLimits(2))*fitZAll(2),'k-.','LineWidth',1);
    for ii = numel(Settings.subjectID):-1:1
        hAx1Plots(ii) = plot(dataZX(dataIdxs(:,ii),ii),dataZY(dataIdxs(:,ii),ii),'LineStyle','none','Marker',lineStyles{ii},'Color',plotColors(ii),'MarkerSize',7,'LineWidth',1);
    end
    axis([xLimits, yLimits],'square');
    xlabel('Norm. Intrinsic Stiffness (x)')
    ylabel('Norm. SOL Back. EMG (x)')
    [hLgd,hLgdIcons] = legend(hAx1Plots,[cellstr(num2str((1:10)'));{'All'}],'Location','NW','FontSize',8,'FontName','Segoe UI','Color','none','Box','off');
    title(hLgd,'Subject:'); hLgd.Title.Visible = 'on'; hLgd.Title.NodeChildren.Position = [0.31 1.05 0];
    for ii = 1:11
        if (ii ~= 11)
            set(hLgdIcons(2*ii+10),'LineStyle','-','Color',plotColors(ii,'light'));
            hLgdIcons(2*ii+11).XData = hLgdIcons(2*ii+11).XData - 0.15;
        end
        hLgdIcons(2*ii+10).XData(2) = hLgdIcons(2*ii+10).XData(2) - 0.3;
        hLgdIcons(ii).Position(1) = hLgdIcons(ii).Position(1) - 0.3;
    end
    set(gca,'TickLength',[0.02 0.025]);
    
    hLgd.Position([1 2]) = [0.897; -0.008];
% export_fig([cd '/Processed/OutcomeFigures/Fig3B_NormalizedCorrelations'],'-pdf','-painters')
    
%% Figure 6 (Point Clouds)
% load([cd '/Processed/statsTLS.mat']);
isPlantTq = DataIntr.tqTarget < -2.5;
xLimits = [-3 3]; yLimits = [-3 3];
xColumn = reshape(repmat({'intrK','reflG'},4,1),[],1);
yColumn = {'solBack','gmBack','glBack','taBack','solRefl','gmRefl','glRefl','taRefl'};
xLabels = reshape(repmat({'Norm. Intrinsic Stiffness (x)','Norm. Reflexive Gain (x)'},4,1),[],1);
yLabels = reshape(repmat({'Norm. Back. EMG (x)','Norm. Refl. EMG (x)'},4,1),[],1);
columnTitles = {'Soleus','Gas. Med.','Gas. Lat.','Tib. Ant.'};

% Z-Score normalization
DataIntrZ.tqTarget = DataIntr.tqTarget;
DataReflZ.tqTarget = DataRefl.tqTarget;
for ii = 2:numel(fieldNamesIntr)
    DataIntrZ.(fieldNamesIntr{ii}) = zscorenan(DataIntr.(fieldNamesIntr{ii}));
    DataReflZ.(fieldNamesRefl{ii}) = zscorenan(DataRefl.(fieldNamesRefl{ii}));
end

% Generation of Figure 6
figure('Units','centimeters','Position',[4 4 16.6 9.5],'Color','w');
subplotOrder = [1,3:4,2,5,7:8,6]; %[1,4,2:3,5,8,6:7];
for ii = 1:8
    hAx(ii) = subplot(2,4,ii);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        plot(xLimits,[0 0],'--','Color',plotColors(10,'light')); plot([0 0],yLimits,'--','Color',plotColors(10,'light'));
        if (ii <= 4)
            dataIdxs = ~isnan(DataIntrZ.(yColumn{ii}));
            fitTLS = zeros(2,2);
            fitTLS(1,2) = tls(detrend(DataIntrZ.(xColumn{ii})(isPlantTq&dataIdxs),'constant'),detrend(DataIntrZ.(yColumn{ii})(isPlantTq&dataIdxs),'constant'));
            fitTLS(1,1) = mean(DataIntrZ.(yColumn{ii})(isPlantTq&dataIdxs)) - fitTLS(1,2)*mean(DataIntrZ.(xColumn{ii})(isPlantTq&dataIdxs));
            fitTLS(2,2) = tls(detrend(DataIntrZ.(xColumn{ii})(~isPlantTq&dataIdxs),'constant'),detrend(DataIntrZ.(yColumn{ii})(~isPlantTq&dataIdxs),'constant'));
            fitTLS(2,1) = mean(DataIntrZ.(yColumn{ii})(~isPlantTq&dataIdxs)) - fitTLS(2,2)*mean(DataIntrZ.(xColumn{ii})(~isPlantTq&dataIdxs));
            plot(xLimits(1):xLimits(2),fitTLS(1,1)+(xLimits(1):xLimits(2))*fitTLS(1,2),'--','Color',plotColors(2),'LineWidth',1,'HandleVisibility','off');
            plot(xLimits(1):xLimits(2),fitTLS(2,1)+(xLimits(1):xLimits(2))*fitTLS(2,2),'Color',plotColors(1),'LineWidth',1,'HandleVisibility','off');
            set(gca,'ColorOrderIndex',1);
            hAxPlot(1) = plot(DataIntrZ.(xColumn{ii})(isPlantTq&dataIdxs),DataIntrZ.(yColumn{ii})(isPlantTq&dataIdxs),'o','MarkerSize',4,'LineWidth',1,'Color',plotColors(2));
            hAxPlot(2) = plot(DataIntrZ.(xColumn{ii})(~isPlantTq&dataIdxs),DataIntrZ.(yColumn{ii})(~isPlantTq&dataIdxs),'.','MarkerSize',12,'LineWidth',1,'Color',plotColors(1));
        else
            dataIdxs = ~isnan(DataReflZ.(yColumn{ii}));
            fitTLS = zeros(2,2);
            fitTLS(1,2) = tls(detrend(DataReflZ.(xColumn{ii})(isPlantTq&dataIdxs),'constant'),detrend(DataReflZ.(yColumn{ii})(isPlantTq&dataIdxs),'constant'));
            fitTLS(1,1) = mean(DataReflZ.(yColumn{ii})(isPlantTq&dataIdxs)) - fitTLS(1,2)*mean(DataReflZ.(xColumn{ii})(isPlantTq&dataIdxs));
            fitTLS(2,2) = tls(detrend(DataReflZ.(xColumn{ii})(~isPlantTq&dataIdxs),'constant'),detrend(DataReflZ.(yColumn{ii})(~isPlantTq&dataIdxs),'constant'));
            fitTLS(2,1) = mean(DataReflZ.(yColumn{ii})(~isPlantTq&dataIdxs)) - fitTLS(2,2)*mean(DataReflZ.(xColumn{ii})(~isPlantTq&dataIdxs));
            plot(xLimits(1):xLimits(2),fitTLS(1,1)+(xLimits(1):xLimits(2))*fitTLS(1,2),'--','Color',plotColors(2),'LineWidth',1,'HandleVisibility','off');
            plot(xLimits(1):xLimits(2),fitTLS(2,1)+(xLimits(1):xLimits(2))*fitTLS(2,2),'Color',plotColors(1),'LineWidth',1,'HandleVisibility','off');
            set(gca,'ColorOrderIndex',1);
            plot(DataReflZ.(xColumn{ii})(isPlantTq&dataIdxs),DataReflZ.(yColumn{ii})(isPlantTq&dataIdxs),'o','MarkerSize',4,'LineWidth',1,'Color',plotColors(2))
            plot(DataReflZ.(xColumn{ii})(~isPlantTq&dataIdxs),DataReflZ.(yColumn{ii})(~isPlantTq&dataIdxs),'.','MarkerSize',12,'LineWidth',1,'Color',plotColors(1))
        end
        axis([xLimits, yLimits],'square');
        xlabel(xLabels{ii});
        if (ii == 1 || ii == 5), ylabel(yLabels{ii}); end
        if (ii < 5), title(columnTitles{ii}); end
        if (ii == 1)
            [hLgd,hLgdIcons] = legend([hAxPlot(2) hAxPlot(1)],{'0 Nm','-5 Nm'},'Location','NW','FontSize',8,'FontName','Segoe UI','Color','none','Box','off');
            hLgd.UserData = hLgdIcons;
            for jj = 1:2
                hLgdIcons(jj).Position(1) = hLgdIcons(jj).Position(1) - 0.2;
            end
            hLgd.Position([1 2]) = [0.025 0.86];
        end
end
for ii = 1:numel(hAx)
    set(hAx(ii),'FontSize',8,'FontName','Segoe UI','Layer','top','TickLength',[0.02 0.025]);
	hAx(ii).Position = hAx(ii).Position + [-0.08+mod((ii-1),4)*0.035 -0.035+(ii<5)*0.02 0.06 0.06];
end
% export_fig([cd '/Processed/OutcomeFigures/Fig6_PointClouds'],'-pdf','-painters')