%% Clean Start
clc, clearvars, close all;

%% Settings
% Overview of conditions to-be processed
Settings.isEMG = 1; % Flag to process EMG (if true)
% Define data storage location
dataCD = cd; fileSepIdxs = strfind(dataCD,filesep);
dataCD = dataCD(1:fileSepIdxs(end)-1);
% Data column/row order settings
Settings.mechDataColumns = [1 6 7 2 3 4 2]; % [-] Columns used to load data [time, posRef, velRef, posMeas, velMeas, tqMeas, Sync]
Settings.EMGDataRows = [1 2 3 4 5 6]; % [-] Rows used to load data [SOL, TA, GM, GL, Sync, Sawtooth]

% Perturbation
Settings.pertAmpl = 0.035; % [rad] Amplitude (peak-to-peak)
% Filter settings
Settings.fcLpPos = 200;  % [Hz] Low-pass filter position for visualization
Settings.nHpEMG  = 2;    % [-]  High-pass filter order for EMG
Settings.fcHpEMG = 5;    % [Hz] High-pass filter frequency for EMG
% Length of (bio)mechanical data segments
Settings.tSegStart = 0.04; % [s] Time before zero/onset
Settings.tSegEnd   = 0.4; % [s] Time after zero/onset
% Length of EMG data segments
Settings.tSegStartEMG = 0.04; % [s] Time before zero/onset
Settings.tSegEndEMG   = 0.1;  % [s] Time after zero/onset
% Create cell array of all perturbation characteristics to be sorted for
Settings.sortFields = {'rise','step'};
% Add folder with all shared functions
addpath(genpath('Functions'));

%% Load & Process data
% Load EMG windows
load([cd '/Processed/EMGWindows.mat']);

for ii = [2 3 1]
    % Define data storage location
    if (ii < 3)
        dataSet = [dataCD '/Data/Subject006/D01_SepStiffnessIdent/Trial4C1R1'];
    else
        dataSet = [dataCD '/Data/Subject006/D01_SepStiffnessIdent/Trial4C3R1'];
    end
    
    % Rise indices for hold-state
    switch ii
        case 1
            jj = 1;
        case 2
            jj = 5;
        case 3
            jj = 2;
    end

    % Load (bio)mechanical and [optionally] EMG data
    [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings);
    % Synchronize EMG and motor data based on position sync.
    [DataEMG.t,DataEMG.syncEMGScaled] = synchronizePosEMG(DataMech.sync,DataEMG.sync,DataMech.t,DataMech.dt,DataEMG.dt);

    % Align and sort the (bio)mechanical data
    MechEns = alignSortMechData(DataMech,Settings);
    % Align and sort the EMG data
    EMGEns = alignSortEMGData(DataMech,DataEMG,MechEns.StretchIdxs,Settings);
    
    % Variance Accounted For/Modeled Torque
        % Model inputs
    dt = 1/2048;
    Settings.neuralDelay = 0.04;
        % Zero-mean (high-pass) filter
    nZeroMeanFilt = 2; % [-]
    fcZeroMeanFilt = 0.033; % [Hz] Ludvig2007a (Sec. IID)
    [Settings.zmFilt.b,Settings.zmFilt.a] = butter(nZeroMeanFilt,(fcZeroMeanFilt*2*dt),'high');
        % Backwards-difference (differentiator) filter
    Settings.backDiffFilt.b = [25/12 -4 3 -4/3 1/4]/dt;
    Settings.backDiffFilt.a = 1;
        % Noise (low-pass) filter
    nLpInFilt = 2; % [-] Ludvig2006 (Sec. IIB)
    fcLpInFilt = 100; % [Hz] Ludvig2007a
    [Settings.lpInFilt.b,Settings.lpInFilt.a] = butter(nLpInFilt,(fcLpInFilt*2*dt),'low');
    
        % Process measurements to input signals (filters equal to online impedance estimation algorithm)
    DataFilt.pos = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.posMeas);
    DataFilt.vel = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.velMeas);
    samplesDelay = (floor(Settings.neuralDelay/DataMech.dt) + 1);
    DataFilt.velRectDelay = [zeros(samplesDelay,1); max(DataFilt.vel(1:end-samplesDelay),0)];
    DataFilt.tq = filter(Settings.zmFilt.b,Settings.zmFilt.a,filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.tqMeas));
    DataFilt.acc = filter(Settings.backDiffFilt.b,Settings.backDiffFilt.a,DataFilt.vel);
    
        % Nonlinear Least Squares solver for activation dynamics parameters
        % Parameter inputs: inital, lower bound and upper bound
    param0 = [10 0.8]; % Ludvig2007a
    lowBound0 = [0 0.01];
    upBound0 = [20 1.5];
        % Create option set with maximum iteration
    options = optimset('lsqnonlin');
    options = optimset(options,'MaxIter',30);
        % Use nonlinear least squares and store results
    paramSolve = lsqnonlin(@vafSimulation,param0,lowBound0,upBound0,options,jj,DataMech,DataSysID,DataFilt,MechEns);
        % Compute VAF via forward-model simulation
    [~,VAFPerc,tqsVAFModel] = vafSimulation(paramSolve,jj,DataMech,DataSysID,DataFilt,MechEns);
        
    % Definition of time vectors
    tEMG = (-size(EMGEns.rise.solBack,1):size(EMGEns.rise.solReflex,1)-1)'*DataEMG.dt;
    
    %% Figure 5 (Stretch Ensembles)
    holdIdxs = find(MechEns.StretchIdxs.step > (DataSysID.holdEndIndices(jj)-60*2048+1) & MechEns.StretchIdxs.step < DataSysID.holdEndIndices(jj));
    % Generation of Figure 5
    figure(100);
    set(gcf,'Units','centimeters','Position',[4 4 16.6 7.5],'Color','w');
    hAx(1) = subplot(2,4,1);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        shadedErrorBar(MechEns.tSeg,mean(MechEns.step.pos(:,holdIdxs),2),std(MechEns.step.pos(:,holdIdxs),[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1)
        hYLabel(1) = ylabel('Position (rad)');
        ylim([-0.02 0.02])
    hAx(3) = subplot(2,4,3);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        hAx3Plots(ii) = shadedErrorBar(tEMG,mean([EMGEns.step.solBack(:,holdIdxs); EMGEns.step.solReflex(:,holdIdxs)],2),std([EMGEns.step.solBack(:,holdIdxs); EMGEns.step.solReflex(:,holdIdxs)],[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1); %#ok<SAGROW>
        hYLabel(3) = ylabel('SOL EMG (xV)');
        ylim([0 175])
    hAx(4) = subplot(2,4,4);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        shadedErrorBar(tEMG,mean([EMGEns.step.gmBack(:,holdIdxs); EMGEns.step.gmReflex(:,holdIdxs)],2),std([EMGEns.step.gmBack(:,holdIdxs); EMGEns.step.gmReflex(:,holdIdxs)],[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1)
        hYLabel(4) = ylabel('GM EMG (xV)');
        ylim([0 175])
    hAx(5) = subplot(2,4,5);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        shadedErrorBar(MechEns.tSeg,mean(MechEns.step.tq(:,holdIdxs) - mean(MechEns.step.tq((MechEns.tSeg < 0),holdIdxs)),2),std(MechEns.step.tq(:,holdIdxs) - mean(MechEns.step.tq((MechEns.tSeg < 0),holdIdxs)),[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1)
        hYLabel(5) = ylabel('Torque (Nm)');
        xlabel('Time (s)')
        ylim([-8 4.75])
    hAx(6) = subplot(2,4,6);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        hAx6Plots(ii,1) = plot(MechEns.tSeg(find(MechEns.tSeg == 0):end),tqsVAFModel(:,1),'Color',plotColors(ii,'light'),'LineWidth',1); %#ok<SAGROW>
        hAx6Plots(ii,2) = plot(MechEns.tSeg(find(MechEns.tSeg == 0):end),tqsVAFModel(:,2),'--','Color',plotColors(ii),'LineWidth',1); %#ok<SAGROW>
        hYLabel(6) = ylabel('Torque (Nm)');
        xlabel('Time (s)')
        text(0,6,'VAF (Steps & Pulses):','FontSize',8,'FontName','Segoe UI')
        ylim([-8 4.75])
    hAx(7) = subplot(2,4,7);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        shadedErrorBar(tEMG,mean([EMGEns.step.glBack(:,holdIdxs); EMGEns.step.glReflex(:,holdIdxs)],2),std([EMGEns.step.glBack(:,holdIdxs); EMGEns.step.glReflex(:,holdIdxs)],[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1)
        hYLabel(8) = ylabel('GL EMG (xV)');
        xlabel('Time (s)')
        ylim([0 175])
    hAx(8) = subplot(2,4,8);
        set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
        shadedErrorBar(tEMG,mean([EMGEns.step.taBack(:,holdIdxs); EMGEns.step.taReflex(:,holdIdxs)],2),std([EMGEns.step.taBack(:,holdIdxs); EMGEns.step.taReflex(:,holdIdxs)],[],2),plotColors(ii),plotColors(ii,'light'),plotColors(ii,'lightest'),'LineWidth',1)
        hYLabel(7) = ylabel('TA EMG (xV)');
        xlabel('Time (s)')
        ylim([0 250])
    legendString{ii} = ['K = ' num2str(DataSysID.holdStates(jj,2),'%.0f') '; G = ' num2str(DataSysID.holdStates(jj,3),'%.0f') '; VAF = ' num2str(VAFPerc,'%.0f') '%']; %#ok<SAGROW>

    if (ii == 1) % Only execute last round of for-loop
        linkaxes([hAx(1) hAx(5)],'x');
        set(hAx(1),'xlim',[-Settings.tSegStart Settings.tSegEnd]);
        linkaxes([hAx(3) hAx(4) hAx(7) hAx(8)],'x');
        set(hAx(3),'xlim',[tEMG(1) 0.1]);
        % Legend manipulations
        [hLgd,hLgdIcons] = legend([hAx3Plots.mainLine],legendString,'FontSize',8,'FontName','Segoe UI','Color','none','Box','off');
        hLgd.UserData = hLgdIcons;
        for jj = 1:3
            hLgdIcons(2*jj+2).XData = hLgdIcons(2*jj+2).XData - [0 0.15];
            hLgdIcons(jj).Position(1) = hLgdIcons(jj).Position(1) - 0.15;
        end
        hLgd.Position([1 2]) = [0.25 0.67];
        [hLgd2,hLgdIcons2] = legend([hAx6Plots(1,1),hAx6Plots(1,2)],{'Measured','Modeled'},'FontSize',8,'FontName','Segoe UI','Color','none','Box','off');
        hLgd2.UserData = hLgdIcons2;
        for jj = 1:2
            hLgdIcons2(2*jj+1).XData = hLgdIcons2(2*jj+1).XData - [0 0.15];
            hLgdIcons2(jj).Position(1) = hLgdIcons2(jj).Position(1) - 0.15;
        end
        set(hLgdIcons2(3),'Color',plotColors(10));
        set(hLgdIcons2(5),'Color','k');
        hLgd2.Position([1 2]) = [0.36 0.39];

        % Plot manipulations
        for jj = [1 3:numel(hAx)]
            set(hAx(jj),'FontSize',8,'FontName','Segoe UI','Layer','top')
            axis(hAx(jj),'square')
            if (jj ~= 6)
                plot(hAx(jj),[0 0],get(hAx(jj),'ylim'),'--','Color',plotColors(10),'LineWidth',1,'HandleVisibility','off');
            end
            uistack(get(hAx(jj),'Children'),'top')
            
            if (any(jj == [1 5 6]))
                xticks(hAx(jj),0:0.1:Settings.tSegEnd)
            else
                xticks(hAx(jj),-Settings.tSegStartEMG:0.04:Settings.tSegEndEMG)
            end
            hAx(jj).TickLength(1) = 0.02;
            if (jj < 5)
                set(hAx(jj),'XTickLabel',[])
            end
            hAx(jj).Position = hAx(jj).Position + [-0.125+mod((jj-1),4)*0.045 -0.035+(jj<5)*0.00 0.1 0.1];
            hYLabel(1).Position(1) = -0.145; hYLabel(5).Position(1) = -0.145;
        end
%         export_fig([cd '/Processed/OutcomeFigures/Fig5_StretchEnsembles'],'-pdf','-painters')
    end
    
    if (ii == 3)
        %% Figure 3A (EMG Window Definitions)
        % Provide typical example from first hold period in Soleus muscle
        holdIdxs = find(MechEns.StretchIdxs.rise > (DataSysID.holdEndIndices(jj)-60*2048+1) & MechEns.StretchIdxs.rise < DataSysID.holdEndIndices(jj));
        solBack = mean(EMGEns.rise.solBack(:,holdIdxs),2);
        solReflex = mean(EMGEns.rise.solReflex(:,holdIdxs),2);
        solEMG = [solBack; solReflex];

        % Generation of Figure 3A
        figure('Units','centimeters','Position',[4 4 7 3.8],'Color','w');
        hAx = axes;
        yLimits = [0 80];
        set(hAx,'ColorOrder',plotColors(1:10)); hold on; box on;
         % Background Patch
        xBack = [-Settings.tSegStartEMG+0.0005 0 0 -Settings.tSegStartEMG+0.0005];
        yBack = [0.15 0.15 mean(solBack) mean(solBack)];
        patch(xBack,yBack,plotColors(1,'lightest'),'EdgeColor',plotColors(1),'LineWidth',1)
         % Full EMG Ensemble
        plot(tEMG,solEMG,'--','Color',plotColors(1),'LineWidth',1)
        xlim([-Settings.tSegStartEMG Settings.tSegEndEMG])
         % Perturbation Onset
        hOnset = plot([0 0],yLimits,'--','Color',plotColors(10),'LineWidth',1);
        uistack(hOnset,'bottom')
    
         % Background Patch (beneath Reflex)
        xBack = [EMGWindows(6).sol(1) EMGWindows(6).sol(2) EMGWindows(6).sol(2) EMGWindows(6).sol(1)]*DataEMG.dt;
        patch(xBack,yBack,plotColors(1,'lightest'),'EdgeColor',plotColors(1),'LineWidth',1)
         % Reflex Patch
        x = [EMGWindows(6).sol(1) (EMGWindows(6).sol(1):EMGWindows(6).sol(2)) EMGWindows(6).sol(2)]*DataEMG.dt;
        y = [mean(solBack) solReflex((EMGWindows(6).sol(1):EMGWindows(6).sol(2))+1,:)' mean(solBack)];
        patch(x,y,plotColors(1,'light'),'EdgeColor',plotColors(1),'LineWidth',1)
    
         % General Settings
        xlabel('Time (s)')
        xticks(-Settings.tSegStartEMG:0.02:Settings.tSegEndEMG)
        ylabel('EMG (nV)')
        ylim(yLimits)
         % Textual Explanations
        text(-Settings.tSegStartEMG/2,11.75,{'Background','EMG \downarrow'},'HorizontalAlignment','center','FontSize',8,'FontName','Segoe UI')
        text((EMGWindows(6).sol(2)-10)*DataEMG.dt,solReflex(EMGWindows(6).sol(2)-9),'\leftarrow Reflex EMG','FontSize',8,'FontName','Segoe UI')
        text(0,yLimits(2)*0.87,{'\leftarrow Perturbation','     Onset'},'FontSize',8,'FontName','Segoe UI')
        set(hAx,'FontSize',8,'FontName','Segoe UI','Layer','top')
        hAx.Position = hAx.Position + [-0.01 -0.04 0.08 0.1];
%         export_fig([cd '/Processed/OutcomeFigures/Fig3A_EMGWindowDefinitions'],'-pdf','-painters')
    end
end