%% Clean Start
clc, clearvars, close all;

%% Settings
% Overview of conditions to-be processed
Settings.isEMG = 1; % Flag to process EMG (if true)
% Define data storage location
dataCD = cd; fileSepIdxs = strfind(dataCD,filesep);
dataCD = dataCD(1:fileSepIdxs(end)-1);
dataSet = [dataCD '/Data/Subject006/D01_SepStiffnessIdent/Trial4C1R1'];
% Data column/row order settings
Settings.mechDataColumns = [1 6 7 2 3 4 2]; % [-] Columns used to load data [time, posRef, velRef, posMeas, velMeas, tqMeas, Sync]
Settings.EMGDataRows = [1 2 3 4 5 6]; % [-] Rows used to load data [SOL, TA, GM, GL, Sync, Sawtooth]

% Perturbation
Settings.pertAmpl = 0.035; % [rad] Amplitude (peak-to-peak)
% Filter settings
Settings.fcLpPos = 200;  % [Hz] Low-pass filter position for visualization
Settings.nHpEMG  = 2;    % [-]  High-pass filter order for EMG
Settings.fcHpEMG = 5;    % [Hz] High-pass filter frequency for EMG
% Length of (bio)mechanical data segments
Settings.tSegStart = 0.04; % [s] Time before zero/onset
Settings.tSegEnd   = 0.4; % [s] Time after zero/onset
% Length of EMG data segments
Settings.tSegStartEMG = 0.04; % [s] Time before zero/onset
Settings.tSegEndEMG   = 0.1;  % [s] Time after zero/onset
% Create cell array of all perturbation characteristics to be sorted for
Settings.sortFields = {'rise'};
% Add folder with all shared functions
addpath(genpath('../Functions'));

%% Load & Process data
% Load EMG windows
load([cd '/Processed/EMGWindows.mat']);

% Load (bio)mechanical and [optionally] EMG data
[DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings);
% Synchronize EMG and motor data based on position sync.
[DataEMG.t,DataEMG.syncEMGScaled] = synchronizePosEMG(DataMech.sync,DataEMG.sync,DataMech.t,DataMech.dt,DataEMG.dt);

% Align and sort the (bio)mechanical data
MechEns = alignSortMechData(DataMech,Settings);
% Align and sort the EMG data
EMGEns = alignSortEMGData(DataMech,DataEMG,MechEns.StretchIdxs,Settings);

% Calculation of low-pass filtered torque
[bTqLpFilt,aTqLpFilt] = dZOHLpFilter(Settings.visFiltSetting*2*pi,1,DataMech.dt);
contTqLp = filter(bTqLpFilt,aTqLpFilt,DataMech.tqMeas);
% Calculation of continuous background and reflex EMG magnitudes
contSolBack = mean(EMGEns.rise.solBack);
% Reflex Measure: RMS(Half-wave-rectification(Reflex response - mean(background)))
contSolReflex = rms(max(EMGEns.rise.solReflex((EMGWindows(6).sol(1):EMGWindows(6).sol(2))+1,:) - mean(EMGEns.rise.solBack),0));

%% Figure 4 (Timeseries Combined)
%%% Figure 4 (Left) (Timeseries Measured Signals)
% Overview of typical timeseries of measured signals during the experiment
% Settings
plotWidth    = 16.6;
plotHeight   = 10;
marginLeft   = 1.49;
marginRight  = 0.1;
marginBottom = 0.82;
marginTop    = 0.35;
nPlotX       = 2;
nPlotY       = 4;
spacingX     = 1.95;
spacingY     = 0.65;
fontSize     = 8;
fontName     = 'Segoe UI';
xLimits = [9 11.1];

% Generation of Figure 4 (Left)
figure('Units','centimeters','Position',[4 4 plotWidth plotHeight],'Color','w');
posSubplots = subplotPositions(plotWidth,plotHeight,marginLeft,marginRight,marginBottom,marginTop,nPlotX,nPlotY,spacingX,spacingY);
hAx(1) = axes('position',posSubplots{7},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    plot(DataMech.t,DataMech.posMeas,'LineWidth',1)
    hYLabel(1) = ylabel({'Position','PF\leftarrow (rad) \rightarrowDF'});
    title('Measured Signals')
hAx(2) = axes('position',posSubplots{5},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
    plot(DataEMG.t,DataEMG.sol,'LineWidth',1);
    plot(DataEMG.t,DataEMG.gm,'LineWidth',1);
    plot(DataEMG.t,DataEMG.gl,'LineWidth',1);
    hYLabel(2) = ylabel({'TS EMG','(nV)'});
    [hLgd,hLgdIcons] = legend({'SOL','GM','GL'},'Orientation','horizontal','FontSize',fontSize,'FontName',fontName,'Color','none','Box','off');
    hLgd.UserData = hLgdIcons;
    for ii = 1:3
        hLgdIcons(2*ii+2).XData = hLgdIcons(2*ii+2).XData - [(ii-1)*0.1 ii*0.1];
        hLgdIcons(ii).Position(1) = hLgdIcons(ii).Position(1) - ii*0.1;
    end
    hLgd.Position([1 2]) = hLgd.Position([1 2]) + [0.033 0.0775];
    ylim([0 100])
hAx(3) = axes('position',posSubplots{3},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
    plot(DataEMG.t,DataEMG.ta,'LineWidth',1)
    hYLabel(3) = ylabel({'TA EMG','(nV)'});
    ylim([0 100])
hAx(4) = axes('position',posSubplots{1},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    set(gca,'ColorOrder',plotColors(1:10)); hold on; box on;
    plot(DataMech.t,DataMech.tqMeas,'LineWidth',1)
    xlabel('Time (s)')
    hYLabel(4) = ylabel({'Torque','PF\leftarrow (Nm) \rightarrowDF'});
    ylim([-7.5 7.5])
linkaxes(hAx,'x');
xlim(xLimits);

% Plot manipulations & Add perturbation onset lines across entire set of subplots
visualStretches = find(MechEns.StretchIdxs.rise > xLimits(1)*2048 & MechEns.StretchIdxs.rise < xLimits(2)*2048);
for ii = 1:numel(hAx)
    if (ii < 4)
        set(hAx(ii),'XTickLabel',[])
    end
    hYLabel(ii).Position(1) = 8.77;
    for jj = 1:numel(visualStretches)
        plot(hAx(ii),[DataMech.t(MechEns.StretchIdxs.rise(visualStretches(jj))) DataMech.t(MechEns.StretchIdxs.rise(visualStretches(jj)))],get(hAx(ii),'ylim'),'--','Color',plotColors(10),'LineWidth',1,'HandleVisibility','off');
    end
end
    
%%% Figure 4 (Right) (Timeseries Processed Signals)
% Overview of typical timeseries of processed signals during the experiment
% Settings
nPlotY   = 5;
spacingY = 0.3;
xLimits = [20 215];
tLimits = [find(DataMech.t < xLimits(1),1,'last') find(DataMech.t > xLimits(2),1,'first')];
% Generation of Figure 4 (Right)
posSubplots = subplotPositions(plotWidth,plotHeight,marginLeft,marginRight,marginBottom,marginTop,nPlotX,nPlotY,spacingX,spacingY);
hAx2(1) = axes('position',posSubplots{10},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    plot(DataMech.t(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),contTqLp(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
        plot(DataMech.t(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),1*ones(numel(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
        plot(DataMech.t(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),-1*ones(numel(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),contTqLp(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),'Color',plotColors(1),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),1*ones(60*2048,1),'--','Color',plotColors(2),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),-1*ones(60*2048,1),'--','Color',plotColors(2),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),contTqLp(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),-4*ones(numel(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),-6*ones(numel(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),contTqLp(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),'Color',plotColors(1),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),-4*ones(60*2048,1),'--','Color',plotColors(2),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),-6*ones(60*2048,1),'--','Color',plotColors(2),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1:tLimits(2)),contTqLp(DataSysID.holdEndIndices(2)+1:tLimits(2)),'Color',plotColors(1,'light'),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(2)+1:tLimits(2)),1*ones(numel(DataSysID.holdEndIndices(2)+1:tLimits(2)),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
        plot(DataMech.t(DataSysID.holdEndIndices(2)+1:tLimits(2)),-1*ones(numel(DataSysID.holdEndIndices(2)+1:tLimits(2)),1),'--','Color',plotColors(2,'light'),'LineWidth',1)
    hYLabel2(1) = ylabel({'Filtered','Torque','(Nm)'});
    title('Processed Signals')
    ylim([-6.5 2])
hAx2(2) = axes('position',posSubplots{8},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    plot(DataMech.t(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),DataSysID.intrStiffness(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),DataSysID.intrStiffness(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),'Color',plotColors(1),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),DataSysID.intrStiffness(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),DataSysID.intrStiffness(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),'Color',plotColors(1),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1:tLimits(2)),DataSysID.intrStiffness(DataSysID.holdEndIndices(2)+1:tLimits(2)),'Color',plotColors(1,'light'),'LineWidth',1)
    hYLabel2(2) = ylabel({'Intrinsic','Stiffness','(Nm/rad)'});
    ylim([0 120])
hAx2(3) = axes('position',posSubplots{6},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= tLimits(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048))),contSolBack(find(MechEns.StretchIdxs.rise >= tLimits(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048)),'Color',plotColors(1,'light'),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)))),contSolBack(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1))),'Color',plotColors(1),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048))),contSolBack(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048)),'Color',plotColors(1,'light'),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)))),contSolBack(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2))),'Color',plotColors(1),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)):find(MechEns.StretchIdxs.rise >= tLimits(2)))),contSolBack(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)):find(MechEns.StretchIdxs.rise >= tLimits(2))),'Color',plotColors(1,'light'),'LineWidth',1)
    hYLabel2(3) = ylabel({'Back.','SOL EMG','(nV)'});
    ylim([0 15])
hAx2(4) = axes('position',posSubplots{4},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    plot(DataMech.t(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),DataSysID.reflexive(tLimits(1):DataSysID.holdEndIndices(1)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),DataSysID.reflexive(DataSysID.holdEndIndices(1)+1-60*2048:DataSysID.holdEndIndices(1)),'Color',plotColors(1),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),DataSysID.reflexive(DataSysID.holdEndIndices(1)+1:DataSysID.holdEndIndices(2)-60*2048),'Color',plotColors(1,'light'),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),DataSysID.reflexive(DataSysID.holdEndIndices(2)+1-60*2048:DataSysID.holdEndIndices(2)),'Color',plotColors(1),'LineWidth',1)
    plot(DataMech.t(DataSysID.holdEndIndices(2)+1:tLimits(2)),DataSysID.reflexive(DataSysID.holdEndIndices(2)+1:tLimits(2)),'Color',plotColors(1,'light'),'LineWidth',1)
    hYLabel2(4) = ylabel({'Reflexive','Gain','(Nm/rad/s)'});
    ylim([2 13])
hAx2(5) = axes('position',posSubplots{2},'ColorOrder',plotColors(1:10),'FontSize',fontSize,'FontName',fontName,'Layer','top'); hold on; box on;
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= tLimits(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048))),contSolReflex(find(MechEns.StretchIdxs.rise >= tLimits(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048)),'Color',plotColors(1,'light'),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)))),contSolReflex(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1))),'Color',plotColors(1),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048))),contSolReflex(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(1)):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048)),'Color',plotColors(1,'light'),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)))),contSolReflex(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)+1-60*2048):find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2))),'Color',plotColors(1),'LineWidth',1)
    stairs(DataMech.t(MechEns.StretchIdxs.rise(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)):find(MechEns.StretchIdxs.rise >= tLimits(2)))),contSolReflex(find(MechEns.StretchIdxs.rise >= DataSysID.holdEndIndices(2)):find(MechEns.StretchIdxs.rise >= tLimits(2))),'Color',plotColors(1,'light'),'LineWidth',1)
    xlabel('Time (s)')
    hYLabel2(5) = ylabel({'Reflex','SOL EMG','(nV)'});
    ylim([0 140])
linkaxes(hAx2,'x');
xlim(xLimits);

% Plot manipulations & Add grey-background patches for 2 'hold periods' across all subplots
for ii = 1:numel(hAx2)
    if (ii < 5)
        set(hAx2(ii),'XTickLabel',[])
    end
    hYLabel2(ii).Position(1) = -1.1;
    yLimits = get(hAx2(ii),'ylim');
    for jj = 1:2
        patch(hAx2(ii),[repmat(DataMech.t(DataSysID.holdEndIndices(jj)+1-60*2048),2,1); repmat(DataMech.t(DataSysID.holdEndIndices(jj)),2,1)],[yLimits(1); repmat(yLimits(2),2,1); yLimits(1)],plotColors(10,'lightest'),'EdgeColor','none','LineWidth',1)
    end
    set(hAx2(ii),'Children',flipud(get(hAx2(ii),'Children')));
end

% export_fig([cd '/Processed/OutcomeFigures/Fig4_TimeseriesSignals'],'-pdf','-painters')