%% Clean Start
clc, clearvars, close all;

%% Settings
% Overview of conditions to-be processed
Settings.isEMG = 1; % Flag to process EMG (if true)
Settings.subjectID = cellstr(num2str((1:10).', '%03d')); % Subject identifier (padded to 3 numbers)
% Define data storage location
dataCD = cd; fileSepIdxs = strfind(dataCD,filesep);
dataCD = dataCD(1:fileSepIdxs(end)-1);
% Data column/row order settings
Settings.mechDataColumns = [1 6 7 2 3 4 2]; % [-] Columns used to load data [time, posRef, velRef, posMeas, velMeas, tqMeas, Sync]
Settings.EMGDataRows = [1 2 3 4 5 6]; % [-] Rows used to load data [SOL, TA, GM, GL, Sync, Sawtooth]

% Perturbation
Settings.pertAmpl = 0.035; % [rad] Amplitude (peak-to-peak)
% Filter settings
Settings.fcLpPos = 200;  % [Hz] Low-pass filter position for visualization
Settings.nHpEMG  = 2;    % [-]  High-pass filter order for EMG
Settings.fcHpEMG = 5;    % [Hz] High-pass filter frequency for EMG
% Length of (bio)mechanical data segments
Settings.tSegStart = 0.04; % [s] Time before zero/onset
Settings.tSegEnd   = 0.4; % [s] Time after zero/onset
% Length of EMG data segments
Settings.tSegStartEMG = 0.04; % [s] Time before zero/onset
Settings.tSegEndEMG   = 0.1; % [s] Time after zero/onset
% Create cell array of all perturbation characteristics to be sorted for
Settings.sortFields = {'rise'};
% Add folder with all shared functions
addpath(genpath('Functions'));

% Model inputs
dt = 1/2048;
Settings.neuralDelay = 0.04;
% Zero-mean (high-pass) filter
nZeroMeanFilt = 2; % [-]
fcZeroMeanFilt = 0.033; % [Hz] Ludvig2007a (Sec. IID)
[Settings.zmFilt.b,Settings.zmFilt.a] = butter(nZeroMeanFilt,(fcZeroMeanFilt*2*dt),'high');
% Backwards-difference (differentiator) filter
Settings.backDiffFilt.b = [25/12 -4 3 -4/3 1/4]/dt;
Settings.backDiffFilt.a = 1;
% Noise (low-pass) filter
nLpInFilt = 2; % [-] Ludvig2006 (Sec. IIB)
fcLpInFilt = 100; % [Hz] Ludvig2007a
[Settings.lpInFilt.b,Settings.lpInFilt.a] = butter(nLpInFilt,(fcLpInFilt*2*dt),'low');

%% Load & Process Data
% Pre-allocate outcome measure struct
fieldNamesVAF = {'paramW','paramZ','VAFPerc','lpTqVar'};
for ii = 1:numel(fieldNamesVAF)
    DataVAF.(fieldNamesVAF{ii}) = zeros(20,numel(Settings.subjectID));
end

%% VAF Analysis
for ii = numel(Settings.subjectID):-1:1
    % Retrieve all experimental folders/files
    dirContent = dir([dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/*.mat']);
    dirContent = {dirContent.name}';
    ll = 1;

    for jj = 1:numel(dirContent)
        dataSet = [dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/' dirContent{jj}];
        % Load (bio)mechanical and [optionally] EMG data
        [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings);
        
        % Ensemble averaging  
        SettingsEns.tSegStart = 0.04; % [s] Time before zero/onset
        SettingsEns.tSegEnd   = 0.4; % [s] Time after zero/onset
        SettingsEns.fcLpPos   = 200;  % [Hz] Low-pass filter position for visualization
        SettingsEns.sortFields = {'rise'};
        MechEns = alignSortMechData(DataMech,SettingsEns);
        
        % Process measurements to input signals (filters equal to online impedance estimation algorithm)
        DataFilt.pos = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.posMeas);
        DataFilt.vel = filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.velMeas);
        samplesDelay = (floor(Settings.neuralDelay/DataMech.dt) + 1);
        DataFilt.velRectDelay = [zeros(samplesDelay,1); max(DataFilt.vel(1:end-samplesDelay),0)];
        DataFilt.tq = filter(Settings.zmFilt.b,Settings.zmFilt.a,filter(Settings.lpInFilt.b,Settings.lpInFilt.a,DataMech.tqMeas));
        DataFilt.acc = filter(Settings.backDiffFilt.b,Settings.backDiffFilt.a,DataFilt.vel);
        
        % Compute VAF after nonlinear optimization of activation dynamics parameters
        for kk = 1:size(DataSysID.holdStates,1)
            % Parameter inputs: inital, lower bound and upper bound
            param0 = [10 0.8]; % Ludvig2007a
            lowBound0 = [0 0.01];
            upBound0 = [20 1.5];
            % Create option set with maximum iteration
            options = optimset('lsqnonlin');
            options = optimset(options,'MaxIter',30);
            % Use nonlinear least squares and store results
            paramSolve = lsqnonlin(@vafSimulation,param0,lowBound0,upBound0,options,kk,DataMech,DataSysID,DataFilt,MechEns);
            DataVAF.paramW(ll,ii) = paramSolve(1); DataVAF.paramZ(ll,ii) = paramSolve(2);
            % Compute VAF via forward-model simulation
            [~,DataVAF.VAFPerc(ll,ii)] = vafSimulation(paramSolve,kk,DataMech,DataSysID,DataFilt,MechEns);
            % Obtain variance of measured torque within hold period
            pertIdxs = MechEns.StretchIdxs.rise(MechEns.StretchIdxs.rise > (DataSysID.holdEndIndices(kk)-60*2048+1) & MechEns.StretchIdxs.rise < DataSysID.holdEndIndices(kk));
            DataVAF.lpTqVar(ll,ii) = var(DataMech.tqMeas(pertIdxs));
            ll = ll + 1;
        end
    end
end

for ii = 1:numel(fieldNamesVAF)
    DataVAF.(fieldNamesVAF{ii})(12,9) = NaN; % Remove incorrect task execution
end

%%% Save Results
% save([cd '/Processed/dataVAF.mat'],'DataVAF','-v7.3');