%% Clean Start
clc, clearvars, close all;

%% Settings
% Overview of conditions to-be processed
Settings.isEMG = 1; % Flag to process EMG (if true)
Settings.subjectID = cellstr(num2str((1:10).', '%03d')); % Subject identifier (padded to 3 numbers)
% Define data storage location
dataCD = cd; fileSepIdxs = strfind(dataCD,filesep);
dataCD = dataCD(1:fileSepIdxs(end)-1);
% Data column/row order settings
Settings.mechDataColumns = [1 6 7 2 3 4 2]; % [-] Columns used to load data [time, posRef, velRef, posMeas, velMeas, tqMeas, Sync]
Settings.EMGDataRows = [1 2 3 4 5 6]; % [-] Rows used to load data [SOL, TA, GM, GL, Sync, Sawtooth]

% Perturbation
Settings.pertAmpl = 0.035; % [rad] Amplitude (peak-to-peak)
% Filter settings
Settings.fcLpPos = 200;  % [Hz] Low-pass filter position for visualization
Settings.nHpEMG  = 2;    % [-]  High-pass filter order for EMG
Settings.fcHpEMG = 5;    % [Hz] High-pass filter frequency for EMG
% Length of (bio)mechanical data segments
Settings.tSegStart = 0.04; % [s] Time before zero/onset
Settings.tSegEnd   = 0.4;  % [s] Time after zero/onset
% Length of EMG data segments
Settings.tSegStartEMG = 0.04; % [s] Time before zero/onset
Settings.tSegEndEMG   = 0.1; % [s] Time after zero/onset
% Create cell array of all perturbation characteristics to be sorted for
Settings.sortFields = {'rise','stepFall'};
% Add folder with all shared functions
addpath(genpath('Functions'));

%% Stretch Analysis
for ii = numel(Settings.subjectID):-1:1
    % Retrieve all experimental folders/files
    dirContent = dir([dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/*.mat']);
    dirContent = {dirContent.name}';
    figure('Name','Dorsiflexion (SOL/GM/GL) and Plantarflexion (TA) EMG responses');

    for jj = 1:numel(dirContent)
        dataSet = [dataCD '/Data/Subject' Settings.subjectID{ii} '/D01_SepStiffnessIdent/' dirContent{jj}];
        
        %%% Load & Process data
        % Load (bio)mechanical and [optionally] EMG data
        [DataMech,DataEMG,DataSysID,Settings] = loadData(dataSet,Settings);
        % Synchronize EMG and motor data based on position sync.
        [DataEMG.t,DataEMG.syncEMGScaled] = synchronizePosEMG(DataMech.sync,DataEMG.sync,DataMech.t,DataMech.dt,DataEMG.dt);

        % Align and sort the (bio)mechanical data
        MechEns = alignSortMechData(DataMech,Settings);
        % Align and sort the EMG data
        EMGEns = alignSortEMGData(DataMech,DataEMG,MechEns.StretchIdxs,Settings);
        
        % Check for EMG responses after dorsiflexion (SOL/GM/GL) and Plantarflexion (TA) perturbations
        hAx(1) = subplot(2,2,1);
            hold on; grid on;
            plot(EMGEns.tSeg,mean(EMGEns.rise.solReflex,2))
            title('Soleus')
            ylabel('EMG [\muV]')
        hAx(2) = subplot(2,2,2);
            hold on; grid on;
            plot(EMGEns.tSeg,mean(EMGEns.stepFall.taReflex,2))
            title('Tib. Ant.')
        hAx(3) = subplot(2,2,3);
            hold on; grid on;
            plot(EMGEns.tSeg,mean(EMGEns.rise.gmReflex,2))
            plot(EMGEns.tSeg,mean(EMGEns.rise.glReflex,2))
            title('Gastroc. Med./Lat.')
            xlabel('Time [s]')
            ylabel('EMG [\muV]')
        subplot(2,2,4)
            hold on; grid on;
            plot(EMGEns.tSeg,EMGEns.rise.syncReflex)
            title('Sync. Position')
            xlabel('Time [s]')
    end
    suptitle('Ensemble Averages (1 line = ensemble of 1 block)')
	
    %%% Generate EMG Window settings
    switch Settings.subjectID{ii} % Specific windows for each subject (equal length, different location)
            case '001'
               EMGWindows(ii).sol = [91 131]; % [-] Samples after trigger for M1 reflex SOL
               EMGWindows(ii).ta = [93 133]; % [-] Samples after trigger for M1 reflex TA
               EMGWindows(ii).ga = [88 128]; % [-] Samples after trigger for M1 reflex GM/GL
            case '002'
               EMGWindows(ii).sol = [93 133];
               EMGWindows(ii).ta = [93 133];
               EMGWindows(ii).ga = [83 123];
            case '003'
               EMGWindows(ii).sol = [86 126];
               EMGWindows(ii).ta = [94 134];
               EMGWindows(ii).ga = [82 122];
            case '004'
               EMGWindows(ii).sol = [70 110];
               EMGWindows(ii).ta = [77 117];
               EMGWindows(ii).ga = [67 107];
            case '005'
               EMGWindows(ii).sol = [94 134];
               EMGWindows(ii).ta = [95 135];
               EMGWindows(ii).ga = [93 133];
            case '006'
               EMGWindows(ii).sol = [81 121];
               EMGWindows(ii).ta = [90 130];
               EMGWindows(ii).ga = [79 119];
            case '007'
               EMGWindows(ii).sol = [84 124];
               EMGWindows(ii).ta = [88 128];
               EMGWindows(ii).ga = [83 123];
            case '008'
               EMGWindows(ii).sol = [80 120];
               EMGWindows(ii).ta = [83 123];
               EMGWindows(ii).ga = [76 116];
            case '009'
               EMGWindows(ii).sol = [81 121];
               EMGWindows(ii).ta = [83 123];
               EMGWindows(ii).ga = [76 116];
            case '010'
               EMGWindows(ii).sol = [83 123];
               EMGWindows(ii).ta = [96 136];
               EMGWindows(ii).ga = [79 119];
    end
    
    % Add EMG windows to ensemble plot
    plot(hAx(1),[EMGWindows(ii).sol(1) EMGWindows(ii).sol(1)]/2048,get(hAx(1),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
    plot(hAx(1),[EMGWindows(ii).sol(2) EMGWindows(ii).sol(2)]/2048,get(hAx(1),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
    plot(hAx(2),[EMGWindows(ii).ta(1) EMGWindows(ii).ta(1)]/2048,get(hAx(2),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
    plot(hAx(2),[EMGWindows(ii).ta(2) EMGWindows(ii).ta(2)]/2048,get(hAx(2),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
    plot(hAx(3),[EMGWindows(ii).ga(1) EMGWindows(ii).ga(1)]/2048,get(hAx(3),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
    plot(hAx(3),[EMGWindows(ii).ga(2) EMGWindows(ii).ga(2)]/2048,get(hAx(3),'ylim'),'--','Color',[0.6 0.6 0.6],'HandleVisibility','off');
end

%% Save Results
% save([cd '/Processed/EMGWindows.mat'],'EMGWindows','-v7.3');